@@ -10,7 +10,6 @@ from folioclient import FolioClient
 from requests import HTTPError
 
 from folio_migration_tools.helper import Helper
-from folio_migration_tools.library_configuration import FolioRelease
 from folio_migration_tools.migration_report import MigrationReport
 from folio_migration_tools.report_blurbs import Blurbs
 from folio_migration_tools.transaction_migration.legacy_loan import LegacyLoan
@@ -85,12 +84,13 @@ class CirculationHelper:
         Returns:
             TransactionResult: _description_
         """
+
         t0_function = time.time()
         data = {
             "itemBarcode": legacy_loan.item_barcode,
             "userBarcode": legacy_loan.patron_barcode,
             "loanDate": legacy_loan.out_date.isoformat(),
-            "servicePointId": self.service_point_id,
+            "servicePointId": legacy_loan.service_point_id,
             "overrideBlocks": {
                 "itemNotLoanableBlock": {"dueDate": legacy_loan.due_date.isoformat()},
                 "patronBlock": {},
@@ -208,15 +208,12 @@ class CirculationHelper:
 
     @staticmethod
     def create_request(
-        folio_client: FolioClient,
-        legacy_request: LegacyRequest,
-        migration_report: MigrationReport,
-        release: FolioRelease,
+        folio_client: FolioClient, legacy_request: LegacyRequest, migration_report: MigrationReport
     ):
         try:
             path = "/circulation/requests"
             url = f"{folio_client.okapi_url}{path}"
-            data = legacy_request.serialize(release)
+            data = legacy_request.serialize()
             data["requestProcessingParameters"] = {
                 "overrideBlocks": {
                     "itemNotLoanableBlock": {
@@ -231,7 +228,7 @@ class CirculationHelper:
             logging.debug(f"POST {req.status_code}\t{url}\t{json.dumps(data)}")
             if str(req.status_code) == "422":
                 message = json.loads(req.text)["errors"][0]["message"]
-                logging.error(f"{message}\t{json.dumps(data)}")
+                logging.error(f"{message}")
                 migration_report.add_general_statistics(message)
                 return False
             else:
@@ -274,7 +271,6 @@ class CirculationHelper:
 
     @staticmethod
     def extend_open_loan(folio_client: FolioClient, loan, extension_due_date, extend_out_date):
-        # TODO: add logging instead of print out
         try:
             loan_to_put = copy.deepcopy(loan)
             del loan_to_put["metadata"]
