@@ -9,6 +9,7 @@ from folio_migration_tools.custom_exceptions import TransformationFieldMappingEr
 from folio_migration_tools.custom_exceptions import TransformationProcessError
 from folio_migration_tools.custom_exceptions import TransformationRecordFailedError
 from folio_migration_tools.helper import Helper
+from folio_migration_tools.library_configuration import FolioRelease
 from folio_migration_tools.marc_rules_transformation.rules_mapper_base import (
     RulesMapperBase,
 )
@@ -18,26 +19,38 @@ from folio_migration_tools.report_blurbs import Blurbs
 
 
 class Conditions:
+    holdings_type_map = {
+        "u": "Unknown",
+        "v": "Multi-part monograph",
+        "x": "Monograph",
+        "y": "Serial",
+    }
+
     def __init__(
         self,
         folio: FolioClient,
         mapper: RulesMapperBase,
         object_type,
+        folio_release: FolioRelease,
         default_call_number_type_name="",
     ):
         self.filter_chars = r"[.,\/#!$%\^&\*;:{}=\-_`~()]"
         self.filter_chars_dop = r"[.,\/#!$%\^&\*;:{}=\_`~()]"
+        self.folio_release: FolioRelease = folio_release
         self.filter_last_chars = r",$"
         self.folio = folio
         self.default_contributor_type = ""
         self.mapper = mapper
         self.ref_data_dicts = {}
-        self.setup_reference_data_for_all()
         if object_type == "bibs":
+            self.setup_reference_data_for_all()
             self.setup_reference_data_for_bibs()
+        elif object_type == "auth":
+            self.setup_reference_data_for_auth()
         else:
+            self.setup_reference_data_for_all()
             self.setup_reference_data_for_items_and_holdings(default_call_number_type_name)
-        self.condition_cache = {}
+        self.condition_cache: dict = {}
 
     def setup_reference_data_for_bibs(self):
         logging.info("Setting up reference data for bib transformation")
@@ -69,17 +82,12 @@ class Conditions:
         self.default_call_number_type = {}
         logging.info("%s\tholding_note_types", len(self.folio.holding_note_types))
         logging.info("%s\tcall_number_types", len(self.folio.call_number_types))
-        self.holdings_types = list(
-            self.folio.folio_get_all("/holdings-types", "holdingsTypes", "", 1000)
-        )
-        logging.info("%s\tholdings types", len(self.holdings_types))
+        self.setup_and_validate_holdings_types()
         # Raise for empty settings
         if not self.folio.holding_note_types:
             raise TransformationProcessError("", "No holding_note_types in FOLIO")
         if not self.folio.call_number_types:
             raise TransformationProcessError("", "No call_number_types in FOLIO")
-        if not self.holdings_types:
-            raise TransformationProcessError("", "No holdings_types in FOLIO")
         if not self.folio.locations:
             raise TransformationProcessError("", "No locations in FOLIO")
 
@@ -104,25 +112,54 @@ class Conditions:
             )
         logging.info("Default Callnumber type Name:\t%s", self.default_call_number_type["name"])
 
+    def setup_and_validate_holdings_types(self):
+        self.holdings_types = list(
+            self.folio.folio_get_all("/holdings-types", "holdingsTypes", "", 1000)
+        )
+        if not self.holdings_types:
+            raise TransformationProcessError("", "No holdings_types in FOLIO")
+        missing_holdings_types = [
+            ht
+            for ht in self.holdings_type_map.values()
+            if ht not in [ht_ref["name"] for ht_ref in self.holdings_types]
+        ]
+        if any(missing_holdings_types):
+            raise TransformationProcessError(
+                "",
+                "Holdings types are missing from the tenant. Please set them up",
+                missing_holdings_types,
+            )
+        logging.info("%s\tholdings types", len(self.holdings_types))
+
     def setup_reference_data_for_all(self):
+
+        logging.info(f"{len(self.folio.class_types)}\tclass_types")
         logging.info(
             f"{len(self.folio.electronic_access_relationships)}\telectronic_access_relationships"
         )
-        logging.info(f"{len(self.folio.class_types)}\tclass_types")
-        self.statistical_codes = list(
-            self.folio.folio_get_all(
-                "/statistical-codes",
-                "statisticalCodes",
-                "?query=cql.allRecords=1",
-                1000,
-            )
-        )
+        self.statistical_codes = self.folio.statistical_codes
         logging.info(f"{len(self.statistical_codes)} \tstatistical_codes")
 
         # Raise for empty settings
         if not self.folio.class_types:
             raise TransformationProcessError("", "No class_types in FOLIO")
 
+    def setup_reference_data_for_auth(self):
+        if self.folio_release not in [FolioRelease.morning_glory, FolioRelease.lotus]:
+            self.authority_source_files = list(
+                self.folio.folio_get_all(
+                    "/authority-source-files", "authoritySourceFiles", self.folio.cql_all, 1000
+                )
+            )
+            logging.info(f"{len(self.authority_source_files)} \tAuthority source files")
+        self.authority_note_types = list(
+            self.folio.folio_get_all(
+                "/authority-note-types", "authorityNoteTypes", self.folio.cql_all, 1000
+            )
+        )
+        logging.info(f"{len(self.authority_note_types)} \tAuthority note types")
+        logging.info(f"{len(self.folio.identifier_types)} \tidentifier types")
+
     def get_condition(
         self, name, legacy_id, value, parameter=None, marc_field: field.Field = None
     ):
@@ -141,6 +178,10 @@ class Conditions:
     def condition_trim(self, legacy_id, value, parameter, marc_field: field.Field):
         return value.strip()
 
+    def condition_set_holdings_type_id(self, legacy_id, value, parameter, marc_field: field.Field):
+        self.mapper.migration_report.add(Blurbs.HoldingsTypeMapping, "Condition in rules hit")
+        return ""
+
     def condition_concat_subfields_by_name(
         self, legacy_id, value, parameter, marc_field: field.Field
     ):
@@ -246,7 +287,14 @@ class Conditions:
             )
             return t[0]
         identifier_type = next(
-            (f for f in self.folio.identifier_types if f["name"] in parameter["names"]),
+            (
+                f
+                for f in self.folio.identifier_types
+                if (
+                    f["name"] in parameter.get("names", "non existant")
+                    or f["name"] in parameter.get("name", "non existant")
+                )
+            ),
             None,
         )
         self.mapper.migration_report.add(Blurbs.MappedIdentifierTypes, identifier_type["name"])
@@ -261,6 +309,18 @@ class Conditions:
 
     def condition_set_holding_note_type_id_by_name(
         self, legacy_id, value, parameter, marc_field: field.Field
+    ):
+        self.mapper.migration_report.add(
+            Blurbs.Exceptions,
+            (
+                "Condition set_holding_note_type_id_by_name is deprecated. "
+                "Use set_holdings_note_type_id instead"
+            ),
+        )
+        return self.condition_set_holdings_note_type_id(legacy_id, value, parameter, marc_field)
+
+    def condition_set_holdings_note_type_id(
+        self, legacy_id, value, parameter, marc_field: field.Field
     ):
         try:
             t = self.get_ref_data_tuple_by_name(
@@ -277,6 +337,24 @@ class Conditions:
                 parameter.get("name", ""),
             ) from ee
 
+    def condition_set_authority_note_type_id(
+        self, legacy_id, _, parameter, marc_field: field.Field
+    ):
+        try:
+            t = self.get_ref_data_tuple_by_name(
+                self.authority_note_types, "authority_note_types", parameter["name"]
+            )
+            self.mapper.migration_report.add(Blurbs.MappedNoteTypes, t[1])
+            return t[0]
+        except Exception as ee:
+            logging.error(ee)
+            raise TransformationProcessError(
+                legacy_id,
+                f'Authority note type mapping error.\tParameter: {parameter.get("name", "")}\t'
+                f"MARC Field: {marc_field}. Is mapping rules and ref data aligned?",
+                parameter.get("name", ""),
+            ) from ee
+
     def condition_set_classification_type_id(
         self, legacy_id, value, parameter, marc_field: field.Field
     ):
@@ -409,43 +487,38 @@ class Conditions:
             if not t:
                 self.mapper.migration_report.add(
                     Blurbs.ContributorTypeMapping,
-                    f'Mapping failed for $e "{subfield}" ({normalized_subfield}) ',
+                    f"Mapping failed for {marc_field.tag} $e {subfield} (Normalized: {normalized_subfield}) ",
                 )
                 Helper.log_data_issue(
                     legacy_id,
-                    "Mapping failed for $e",
+                    f"Mapping failed for {marc_field.tag} $e",
                     f'{subfield}" ({normalized_subfield}) ',
                 )
             else:
                 self.mapper.migration_report.add(
                     Blurbs.ContributorTypeMapping,
-                    f'Contributor type name {t[1]} found for $e "{normalized_subfield}" ({subfield}) ',
+                    f"Contributor type name {t[1]} found for {marc_field.tag} $e {normalized_subfield} ({subfield}) ",
                 )
                 return t[0]
         return self.default_contributor_type["id"]
 
-    def condition_set_instance_id_by_map(
-        self, legacy_id, value, parameter, marc_field: field.Field
-    ):
-        try:
-            if value:
-                if value.strip() not in self.mapper.instance_id_map:
-                    raise ValueError()
-                return self.mapper.instance_id_map[value.strip()]["folio_id"]
-            Helper.log_data_issue("", "No instance id provided", marc_field.format_field())
-            return ""
-        except Exception:
-            raise TransformationRecordFailedError(
-                legacy_id,
-                "Old instance id not in map",
-                f"{marc_field.format_field()}",
-            )
-
     def condition_set_url_relationship(self, legacy_id, value, parameter, marc_field: field.Field):
         return self._extracted_from_condition_set_electronic_access_relations_id_2("8", marc_field)
 
     def condition_set_call_number_type_by_indicator(
         self, legacy_id, value, parameter, marc_field: pymarc.Field
+    ):
+        self.mapper.migration_report.add(
+            Blurbs.Exceptions,
+            (
+                "Condition set_call_number_type_by_indicator is deprecated. "
+                "Change to set_call_number_type_id"
+            ),
+        )
+        return self.condition_set_call_number_type_id(legacy_id, value, parameter, marc_field)
+
+    def condition_set_call_number_type_id(
+        self, legacy_id, value, parameter, marc_field: pymarc.Field
     ):
         first_level_map = {
             "0": "Library of Congress classification",
@@ -521,23 +594,6 @@ class Conditions:
                 f"Alternative title type not found for {parameter['name']} {marc_field}",
             )
 
-    def setup_location_code_from_second_column(self):
-        try:
-            other_columns = [
-                f for f in self.mapper.location_map[0].keys() if f not in ["folio_code"]
-            ]
-            if len(other_columns) > 1:
-                raise TransformationProcessError(
-                    "",
-                    "Other location map columns could not be used since more than one",
-                    other_columns,
-                )
-            elif len(other_columns) == 1:
-                logging.info(f"{other_columns[0]} will be used for location mapping")
-                return {lm[other_columns[0]]: lm["folio_code"] for lm in self.mapper.location_map}
-        except Exception as ee:
-            raise TransformationProcessError("", f"{ee}", self.mapper.location_map)
-
     def condition_set_location_id_by_code(
         self, legacy_id, value, parameter, marc_field: field.Field
     ):
@@ -553,29 +609,46 @@ class Conditions:
     def condition_set_permanent_location_id(
         self, legacy_id, value, parameter, marc_field: field.Field
     ):
-        # Setup mapping if not already set up
+
         if "legacy_locations" not in self.ref_data_dicts:
             try:
                 d = {lm["legacy_code"]: lm["folio_code"] for lm in self.mapper.location_map}
                 self.ref_data_dicts["legacy_locations"] = d
+                for folio_code in d.values():
+                    t = self.get_ref_data_tuple_by_code(
+                        self.folio.locations, "locations", folio_code
+                    )
+                    if not t:
+                        raise TransformationProcessError(
+                            "", "No FOLIO location found for code", folio_code
+                        )
+                if "*" not in d:
+                    raise TransformationProcessError(
+                        "",
+                        (
+                            "Fallback location mapping missing. Add a row with a * in the "
+                            "legacy_code column and a location code to map unmapped locations to"
+                        ),
+                        "",
+                    )
             except KeyError as ke:
                 if "folio_code" in str(ke):
                     raise TransformationProcessError(
                         legacy_id, "Your location map lacks the column folio_code"
-                    )
+                    ) from ke
                 if "legacy_code" in str(ke):
-                    logging.info(
-                        "legacy_code column not found. "
-                        "Trying to use other columns from location map."
-                    )
-                    self.ref_data_dicts[
-                        "legacy_locations"
-                    ] = self.setup_location_code_from_second_column()
+                    raise TransformationProcessError(
+                        legacy_id, "Your location map lacks the column legacy_code"
+                    ) from ke
+
         # Get the right code from the location map
-        if self.mapper.location_map and any(self.mapper.location_map):
-            mapped_code = self.ref_data_dicts["legacy_locations"].get(value.strip(), "").strip()
-        else:  # IF there is no map, assume legacy code is the same as FOLIO code
-            mapped_code = value.strip()
+        mapped_code = self.ref_data_dicts["legacy_locations"].get(value.strip(), "").strip()
+        if not mapped_code:
+            mapped_code = self.ref_data_dicts["legacy_locations"].get("*", "").strip()
+            if mapped_code:
+                self.mapper.migration_report.add(
+                    Blurbs.LocationMapping, f"Fallback mapping: {value}->{mapped_code}"
+                )
         # Get the FOLIO UUID for the code and return it
         t = self.get_ref_data_tuple_by_code(self.folio.locations, "locations", mapped_code)
         if not t:
