@@ -2,8 +2,11 @@ import datetime
 import json
 import logging
 import time
+import urllib.parse
 import uuid
+from abc import abstractmethod
 from textwrap import wrap
+from typing import List
 
 import pymarc
 from folio_uuid.folio_uuid import FOLIONamespaces
@@ -17,8 +20,10 @@ from folio_migration_tools.custom_exceptions import TransformationFieldMappingEr
 from folio_migration_tools.custom_exceptions import TransformationProcessError
 from folio_migration_tools.custom_exceptions import TransformationRecordFailedError
 from folio_migration_tools.helper import Helper
+from folio_migration_tools.library_configuration import FileDefinition
 from folio_migration_tools.library_configuration import LibraryConfiguration
 from folio_migration_tools.mapper_base import MapperBase
+from folio_migration_tools.marc_rules_transformation.hrid_handler import HRIDHandler
 from folio_migration_tools.report_blurbs import Blurbs
 
 
@@ -27,23 +32,32 @@ class RulesMapperBase(MapperBase):
         self,
         folio_client: FolioClient,
         library_configuration: LibraryConfiguration,
+        task_configuration,
+        schema: dict,
         conditions=None,
+        parent_id_map: dict[str, tuple] = None,
     ):
-        super().__init__(library_configuration, folio_client)
+        super().__init__(library_configuration, folio_client, parent_id_map)
         self.parsed_records = 0
+        self.id_map: dict[str, tuple] = {}
         self.start = time.time()
         self.last_batch_time = time.time()
         self.folio_client: FolioClient = folio_client
-        self.holdings_json_schema = self.fetch_holdings_schema()
-        self.instance_json_schema = self.get_instance_schema()
-        self.schema = {}
+        self.schema: dict = schema
+        self.task_configuration = task_configuration
         self.conditions = conditions
         self.item_json_schema = ""
-        self.mappings = {}
+        self.mappings: dict = {}
         self.schema_properties = None
+        if hasattr(self.task_configuration, "hrid_handling"):
+            self.hrid_handler = HRIDHandler(
+                folio_client,
+                self.task_configuration.hrid_handling,
+                self.migration_report,
+                self.task_configuration.deactivate035_from001,
+            )
         logging.info("Current user id is %s", self.folio_client.current_user)
 
-    # TODO: Rebuild and move
     def print_progress(self):
         self.parsed_records += 1
         num_recs = 5000
@@ -58,6 +72,10 @@ class RulesMapperBase(MapperBase):
             )
             self.last_batch_time = time.time()
 
+    @abstractmethod
+    def get_legacy_ids(self, marc_record: Record, idx: int):
+        raise NotImplementedError()
+
     @staticmethod
     def dedupe_rec(rec, props_to_not_dedupe=None):
         if props_to_not_dedupe is None:
@@ -91,7 +109,6 @@ class RulesMapperBase(MapperBase):
             marc_field = self.remove_repeated_subfields(marc_field)
         if has_conditions(mapping):
             values = self.apply_rules(marc_field, mapping, legacy_ids)
-            # TODO: add condition to customize this hardcoded thing
             if marc_field.tag == "655":
                 values[0] = f"Genre: {values[0]}"
             self.add_value_to_target(folio_record, target, values)
@@ -127,6 +144,12 @@ class RulesMapperBase(MapperBase):
                     marc_record["005"].data,
                 )
 
+    @abstractmethod
+    def parse_record(
+        self, marc_record: Record, file_def: FileDefinition, legacy_ids: List[str]
+    ) -> dict:
+        raise NotImplementedError()
+
     @staticmethod
     def use_008_for_dates(marc_record: Record, folio_object: dict, legacy_ids):
         try:
@@ -166,6 +189,109 @@ class RulesMapperBase(MapperBase):
             value1 = marc_field.format_field() if marc_field else ""
             return self.apply_rule(legacy_id, value1, condition_types, marc_field, parameter)
 
+    def process_marc_field(
+        self,
+        folio_record: dict,
+        marc_field: Field,
+        ignored_subsequent_fields,
+        legacy_ids,
+    ):
+        if marc_field.tag == "880":
+            mappings = self.perform_proxy_mapping(marc_field)
+        else:
+            tags_to_ignore = {"880", "001", "008"}
+            mappings = (
+                self.mappings.get(marc_field.tag, {})
+                if marc_field.tag not in tags_to_ignore
+                else []
+            )
+        if mappings:
+            try:
+                self.map_field_according_to_mapping(marc_field, mappings, folio_record, legacy_ids)
+                if any(m.get("ignoreSubsequentFields", False) for m in mappings):
+                    ignored_subsequent_fields.add(marc_field.tag)
+            except TransformationFieldMappingError as tre:
+                tre.log_it()
+                logging.error(
+                    "map_field_according_to_mapping %s %s %s %s %s",
+                    "-".join(legacy_ids),
+                    marc_field.tag,
+                    marc_field.format_field(),
+                    json.dumps(mappings),
+                    tre.message,
+                )
+            except Exception as ee:
+                logging.error(
+                    "map_field_according_to_mapping %s %s %s",
+                    marc_field.tag,
+                    marc_field.format_field(),
+                    json.dumps(mappings),
+                )
+                raise ee
+
+    def perform_proxy_mapping(self, marc_field):
+        proxy_mapping = next(iter(self.mappings.get("880", [])), [])
+        if "6" not in marc_field:
+            self.migration_report.add(Blurbs.Field880Mappings, "Records without $6")
+            return None
+        if not proxy_mapping or not proxy_mapping.get("fieldReplacementBy3Digits", False):
+            return None
+        if not marc_field["6"][:3] or len(marc_field["6"][:3]) != 3:
+            self.migration_report.add(
+                Blurbs.Field880Mappings, "Records with unexpected length in $6"
+            )
+            return None
+        first_three = marc_field["6"][:3]
+
+        target_field = next(
+            (
+                r.get("targetField", "")
+                for r in proxy_mapping.get("fieldReplacementRule", [])
+                if r["sourceDigits"] == first_three
+            ),
+            first_three,
+        )
+        self.migration_report.add(
+            Blurbs.Field880Mappings,
+            f"Source digits: {marc_field['6']} Target field: {target_field}",
+        )
+        mappings = self.mappings.get(target_field, {})
+        if not mappings:
+            self.migration_report.add(
+                Blurbs.Field880Mappings,
+                f"Mapping not set up for target field: {target_field} ({marc_field['6']})",
+            )
+        return mappings
+
+    def report_marc_stats(
+        self, marc_field: Field, bad_tags, legacy_ids, ignored_subsequent_fields
+    ):
+        self.migration_report.add(Blurbs.Trivia, "Total number of Tags processed")
+        self.report_source_and_links(marc_field)
+        self.report_bad_tags(marc_field, bad_tags, legacy_ids)
+        mapped = marc_field.tag in self.mappings
+        if marc_field.tag in ignored_subsequent_fields:
+            mapped = False
+        self.report_legacy_mapping(marc_field.tag, True, mapped)
+
+    def report_source_and_links(self, marc_field: Field):
+        for subfield_2 in marc_field.get_subfields("2"):
+            self.migration_report.add(
+                Blurbs.AuthoritySources, f"Source of heading or term: {subfield_2.split(' ')[0]}"
+            )
+        for subfield_0 in marc_field.get_subfields("0"):
+            code = ""
+            if "(" in subfield_0 and ")" in subfield_0:
+                code = subfield_0[subfield_0.find("(") + 1 : subfield_0.find(")")]
+                code = code.split(" ")[0]
+            elif url := urllib.parse.urlparse(subfield_0):
+                if url.hostname:
+                    code = subfield_0[: subfield_0.find(url.path)]
+            if code:
+                self.migration_report.add(
+                    Blurbs.AuthoritySources, f"$0 base uri or source code: {code}"
+                )
+
     def apply_rules(self, marc_field: pymarc.Field, mapping, legacy_ids):
         try:
             values = []
@@ -204,6 +330,17 @@ class RulesMapperBase(MapperBase):
         except Exception as exception:
             self.handle_generic_exception(self.parsed_records, exception)
 
+    def report_bad_tags(self, marc_field, bad_tags, legacy_ids):
+        if (
+            (not marc_field.tag.isnumeric())
+            and marc_field.tag != "LDR"
+            and marc_field.tag not in bad_tags
+        ):
+            self.migration_report.add(Blurbs.NonNumericTagsInRecord, marc_field.tag)
+            message = "Non-numeric tags in records"
+            Helper.log_data_issue(legacy_ids, message, marc_field.tag)
+            bad_tags.add(marc_field.tag)
+
     def add_value_to_target(self, rec, target_string, value):
         if not value:
             return
@@ -277,10 +414,10 @@ class RulesMapperBase(MapperBase):
         sch = self.schema["properties"]
 
         if not target_string or target_string not in sch:
-            raise TransformationProcessError(
+            raise TransformationFieldMappingError(
                 "",
                 (
-                    f"Target string {target_string} not in Schema! "
+                    f"Target string '{target_string}' not in Schema! "
                     "Check mapping file against the schema. "
                     f"Target type: {sch.get(target_string,{}).get('type','')} Value: {value}"
                 ),
@@ -310,6 +447,16 @@ class RulesMapperBase(MapperBase):
                 ),
             )
 
+    def remove_from_id_map(self, former_ids: List[str]):
+        """removes the ID from the map in case parsing failed
+
+        Args:
+            former_ids (_type_): _description_
+        """
+        for former_id in [id for id in former_ids if id]:
+            if former_id in self.id_map:
+                del self.id_map[former_id]
+
     def create_entity(self, entity_mappings, marc_field, entity_parent_key, index_or_legacy_id):
         entity = {}
         for entity_mapping in entity_mappings:
@@ -353,6 +500,7 @@ class RulesMapperBase(MapperBase):
                         "primary",
                         "isbnValue",
                         "issnValue",
+                        "authorityId",
                     ]
                 )
                 or e_parent in ["electronicAccess", "publication"]
@@ -373,6 +521,20 @@ class RulesMapperBase(MapperBase):
                 # Experimental
                 # self.add_entity_to_record(entity, e_parent, rec, self.schema)
 
+    def handle_suppression(
+        self, folio_record, file_def: FileDefinition, only_discovery_suppress: bool = False
+    ):
+        folio_record["discoverySuppress"] = file_def.suppressed
+        self.migration_report.add(
+            Blurbs.Suppression,
+            f'Suppressed from discovery = {folio_record["discoverySuppress"]}',
+        )
+        if not only_discovery_suppress:
+            folio_record["staffSuppress"] = file_def.staff_suppressed
+            self.migration_report.add(
+                Blurbs.Suppression, f'Staff suppressed = {folio_record["staffSuppress"]} '
+            )
+
     def create_preceding_succeeding_titles(self, entity, e_parent, identifier):
         self.migration_report.add(Blurbs.PrecedingSuccedingTitles, f"{e_parent} created")
         # TODO: Make these uuids deterministic
@@ -399,12 +561,19 @@ class RulesMapperBase(MapperBase):
                     "value": new_entity.get("issnValue"),
                 }
             )
-        logging.log(25, f"{e_parent}\t{json.dumps(new_entity)}")
+        self.extradata_writer.write(e_parent, new_entity)
 
     def apply_rule(self, legacy_id, value, condition_types, marc_field, parameter):
         v = value
         for condition_type in iter(condition_types):
-            v = self.conditions.get_condition(condition_type, legacy_id, v, parameter, marc_field)
+            try:
+                v = self.conditions.get_condition(
+                    condition_type, legacy_id, v, parameter, marc_field
+                )
+            except AttributeError as attr_error:
+                raise TransformationProcessError(
+                    legacy_id, attr_error, condition_type
+                ) from attr_error
         return v
 
     @staticmethod
@@ -418,24 +587,6 @@ class RulesMapperBase(MapperBase):
         else:
             rec[entity_parent_key] = entity
 
-    @staticmethod
-    def get_instance_schema():
-        logging.info("Fetching Instance schema...")
-        instance_schema = FolioClient.get_latest_from_github(
-            "folio-org", "mod-inventory-storage", "ramls/instance.json"
-        )
-        logging.info("done")
-        return instance_schema
-
-    @staticmethod
-    def fetch_holdings_schema():
-        logging.info("Fetching HoldingsRecord schema...")
-        holdings_record_schema = FolioClient.get_latest_from_github(
-            "folio-org", "mod-inventory-storage", "ramls/holdingsrecord.json"
-        )
-        logging.info("done")
-        return holdings_record_schema
-
     @staticmethod
     def grouped(marc_field: Field):
         """Groups the subfields
@@ -448,19 +599,17 @@ class RulesMapperBase(MapperBase):
         Returns:
             _type_: _description_
         """
-        unique_subfields = []
-        repeated_subfields = []
-        results = list()
+        unique_subfields: list = []
+        repeated_subfields: list = []
+        results = []
         for sf, sf_vals in marc_field.subfields_as_dict().items():
             if len(sf_vals) == 1:
                 unique_subfields.extend([sf, sf_vals[0]])
             else:
-                for sf_val in sf_vals:
-                    repeated_subfields.append([sf, sf_val])
+                repeated_subfields.extend([sf, sf_val] for sf_val in sf_vals)
         if any(repeated_subfields):
             for repeated_subfield in repeated_subfields:
-                new_subfields = [repeated_subfield[0], repeated_subfield[1]]
-                new_subfields.extend(unique_subfields)
+                new_subfields = [repeated_subfield[0], repeated_subfield[1], *unique_subfields]
                 temp_field = Field(
                     tag=marc_field.tag,
                     indicators=marc_field.indicators,
@@ -503,7 +652,7 @@ class RulesMapperBase(MapperBase):
         folio_client: FolioClient,
         marc_record: Record,
         folio_record,
-        legacy_id: str,
+        legacy_ids: List[str],
         suppress: bool,
     ):
         """Saves the source Marc_record to the Source record Storage module
@@ -514,10 +663,10 @@ class RulesMapperBase(MapperBase):
             folio_client (FolioClient): _description_
             marc_record (Record): _description_
             folio_record (_type_): _description_
-            legacy_id (str): _description_
+            legacy_ids (List[str]): _description_
             suppress (bool): _description_
         """
-        srs_id = RulesMapperBase.create_srs_id(record_type, folio_client.okapi_url, legacy_id)
+        srs_id = RulesMapperBase.create_srs_id(record_type, folio_client.okapi_url, legacy_ids[-1])
 
         marc_record.add_ordered_field(
             Field(
@@ -551,17 +700,50 @@ class RulesMapperBase(MapperBase):
         srs_types = {
             FOLIONamespaces.holdings: FOLIONamespaces.srs_records_holdingsrecord,
             FOLIONamespaces.instances: FOLIONamespaces.srs_records_bib,
-            FOLIONamespaces.athorities: FOLIONamespaces.srs_records_auth,
+            FOLIONamespaces.authorities: FOLIONamespaces.srs_records_auth,
             FOLIONamespaces.edifact: FOLIONamespaces.srs_records_edifact,
         }
 
-        return str(
-            FolioUUID(
-                okapi_url,
-                srs_types.get(record_type),
-                str(legacy_id),
+        return str(FolioUUID(okapi_url, srs_types.get(record_type), legacy_id))
+
+    @staticmethod
+    def get_bib_id_from_907y(marc_record: Record, index_or_legacy_id):
+        try:
+            return list(set(marc_record["907"].get_subfields("a", "y")))
+        except Exception as e:
+            raise TransformationRecordFailedError(
+                index_or_legacy_id,
+                (
+                    "907 $y and $a is missing is missing, although they is "
+                    "required for this legacy ILS choice"
+                ),
+                marc_record.as_json(),
+            ) from e
+
+    @staticmethod
+    def get_bib_id_from_990a(marc_record: Record, index_or_legacy_id):
+        res = {f["a"].strip() for f in marc_record.get_fields("990") if "a" in f}
+        if marc_record["001"].format_field().strip():
+            res.add(marc_record["001"].format_field().strip())
+        if any(res):
+            return list(res)
+        else:
+            raise TransformationRecordFailedError(
+                index_or_legacy_id,
+                "neither 990$a or 001 found in record.",
+                marc_record.as_json(),
             )
-        )
+
+    @staticmethod
+    def get_bib_id_from_001(marc_record: Record, index_or_legacy_id):
+        try:
+            return [marc_record["001"].format_field().strip()]
+        except Exception as e:
+            raise TransformationRecordFailedError(
+                index_or_legacy_id,
+                "001 is missing, although it is required for Voyager migrations",
+                marc_record.as_json(),
+            ) from e
 
     @staticmethod
     def get_srs_string(
@@ -575,20 +757,23 @@ class RulesMapperBase(MapperBase):
         record_types = {
             FOLIONamespaces.holdings: "MARC_HOLDING",
             FOLIONamespaces.instances: "MARC_BIB",
-            FOLIONamespaces.athorities: "MARC_AUTHORITY",
+            FOLIONamespaces.authorities: "MARC_AUTHORITY",
             FOLIONamespaces.edifact: "EDIFACT",
         }
 
         id_holders = {
             FOLIONamespaces.instances: {
                 "instanceId": folio_object["id"],
-                "instanceHrid": folio_object["hrid"],
+                "instanceHrid": folio_object.get("hrid", ""),
             },
             FOLIONamespaces.holdings: {
                 "holdingsId": folio_object["id"],
-                "holdingsHrid": folio_object["hrid"],
+                "holdingsHrid": folio_object.get("hrid", ""),
+            },
+            FOLIONamespaces.authorities: {
+                "authorityId": folio_object["id"],
+                "authorityHrid": marc_record["001"].data,
             },
-            FOLIONamespaces.athorities: {},
             FOLIONamespaces.edifact: {},
         }
 
