@@ -1,29 +1,25 @@
 """The default mapper, responsible for parsing MARC21 records acording to the
 FOLIO community specifications"""
-import json
 import logging
 import sys
 import time
 import typing
 import uuid
+from pathlib import Path
 from typing import Generator
 from typing import List
-from typing import Set
 
 import pymarc
-import requests
 from defusedxml.ElementTree import fromstring
 from folio_uuid.folio_namespaces import FOLIONamespaces
 from folio_uuid.folio_uuid import FolioUUID
 from folioclient import FolioClient
-from pymarc import Field
 from pymarc.record import Record
 
 from folio_migration_tools.custom_exceptions import TransformationProcessError
 from folio_migration_tools.custom_exceptions import TransformationRecordFailedError
 from folio_migration_tools.helper import Helper
 from folio_migration_tools.library_configuration import FileDefinition
-from folio_migration_tools.library_configuration import HridHandling
 from folio_migration_tools.library_configuration import IlsFlavour
 from folio_migration_tools.library_configuration import LibraryConfiguration
 from folio_migration_tools.marc_rules_transformation.conditions import Conditions
@@ -46,37 +42,24 @@ class BibsRulesMapper(RulesMapperBase):
         super().__init__(
             folio_client,
             library_configuration,
-            Conditions(folio_client, self, "bibs"),
+            task_configuration,
+            self.get_instance_schema(),
+            Conditions(folio_client, self, "bibs", library_configuration.folio_release),
         )
-        self.folio = folio_client
-        self.task_configuration = task_configuration
-        self.record_status = {}
-        self.unique_001s: Set[str] = set()
-        self.holdings_map = {}
-        self.id_map = {}
-        self.srs_recs = []
-        self.schema = self.instance_json_schema
-        self.contrib_name_types = {}
-        self.mapped_folio_fields = {}
-        self.unmapped_folio_fields = {}
-        self.alt_title_map = {}
-        logging.info(f"HRID handling is set to: '{self.task_configuration.hrid_handling}'")
-        self.hrid_handling: HridHandling = self.task_configuration.hrid_handling
         logging.info("Fetching mapping rules from the tenant")
         rules_endpoint = "/mapping-rules/marc-bib"
-        self.mappings = self.folio.folio_get_single_object(rules_endpoint)
+        self.mappings = self.folio_client.folio_get_single_object(rules_endpoint)
         logging.info("Fetching valid language codes...")
         self.language_codes = list(self.fetch_language_codes())
-        self.unmapped_tags = {}
-        self.unmapped_conditions = {}
-        self.instance_relationships = {}
-        self.instance_relationship_types = {}
-        self.other_mode_of_issuance_id = get_unspecified_mode_of_issuance(self.folio)
+        self.instance_relationships: dict = {}
+        self.instance_relationship_types: dict = {}
+        self.other_mode_of_issuance_id = get_unspecified_mode_of_issuance(self.folio_client)
+
         self.start = time.time()
 
     def perform_initial_preparation(self, marc_record: pymarc.Record, legacy_ids):
         folio_instance = {
-            "metadata": self.folio.get_metadata_construct(),
+            "metadata": self.folio_client.get_metadata_construct(),
         }
         folio_instance["id"] = str(
             FolioUUID(
@@ -85,28 +68,45 @@ class BibsRulesMapper(RulesMapperBase):
                 str(legacy_ids[-1]),
             )
         )
-        leader_05 = marc_record.leader[5]
-        self.migration_report.add(Blurbs.RecordStatus, leader_05 or "Empty")
-        self.handle_hrid(folio_instance, marc_record, legacy_ids)
-        if leader_05 == "d":
-            Helper.log_data_issue(legacy_ids, "d in leader. Is this correct?", marc_record.leader)
+        self.hrid_handler.handle_hrid(
+            FOLIONamespaces.instances,
+            folio_instance,
+            marc_record,
+            legacy_ids,
+        )
+        self.handle_leader_05(marc_record, legacy_ids)
+        if self.task_configuration.add_administrative_notes_with_legacy_ids:
+            for legacy_id in legacy_ids:
+                self.add_legacy_id_to_admin_note(folio_instance, legacy_id)
+
         return folio_instance
 
-    def parse_bib(self, legacy_ids, marc_record: pymarc.Record, file_def: FileDefinition):
+    def handle_leader_05(self, marc_record, legacy_ids):
+        leader_05 = marc_record.leader[5] or "Empty"
+        self.migration_report.add(Blurbs.RecordStatus, f"Original value: {leader_05}")
+        if leader_05 not in ["a", "c", "d", "n", "p"]:
+            marc_record.leader = f"{marc_record.leader[:5]}c{marc_record.leader[6:]}"
+            self.migration_report.add(Blurbs.RecordStatus, f"Changed {leader_05} to c")
+        if leader_05 == "d":
+            Helper.log_data_issue(legacy_ids, "d in leader. Is this correct?", marc_record.leader)
+
+    def parse_record(
+        self, marc_record: pymarc.Record, file_def: FileDefinition, legacy_ids: List[str]
+    ) -> dict:
         """Parses a bib recod into a FOLIO Inventory instance object
         Community mapping suggestion: https://bit.ly/2S7Gyp3
          This is the main function
 
         Args:
-            legacy_ids (_type_): _description_
             marc_record (pymarc.Record): _description_
             file_def (FileDefinition): _description_
+            legacy_ids (List[str]): List of legacy ids in record
 
         Returns:
-            _type_: _description_
+            dict: _description_
         """
         self.print_progress()
-        ignored_subsequent_fields = set()
+        ignored_subsequent_fields: set = set()
         bad_tags = set(self.task_configuration.tags_to_delete)  # "907"
         folio_instance = self.perform_initial_preparation(marc_record, legacy_ids)
         for marc_field in marc_record:
@@ -125,74 +125,9 @@ class BibsRulesMapper(RulesMapperBase):
         )
         self.dedupe_rec(clean_folio_instance)
         marc_record.remove_fields(*list(bad_tags))
-        self.report_folio_mapping(clean_folio_instance, self.instance_json_schema)
-        if clean_folio_instance["discoverySuppress"]:
-            self.migration_report.add_general_statistics("Suppressed from discovery")
-        # TODO: trim away multiple whitespace and newlines..
-        # TODO: createDate and update date and catalogeddate
+        self.report_folio_mapping(clean_folio_instance, self.schema)
         return clean_folio_instance
 
-    def process_marc_field(
-        self,
-        folio_instance,
-        marc_field,
-        ignored_subsequent_fields,
-        legacy_ids,
-    ):
-        if marc_field.tag == "880" and "6" in marc_field:
-            mappings = self.perform_proxy_mapping(marc_field)
-        else:
-            tags_to_ignore = {"880", "001", "008"}
-            mappings = (
-                self.mappings.get(marc_field.tag, {})
-                if marc_field.tag not in tags_to_ignore
-                else []
-            )
-        if mappings:
-            try:
-                self.map_field_according_to_mapping(
-                    marc_field, mappings, folio_instance, legacy_ids
-                )
-                if any(m.get("ignoreSubsequentFields", False) for m in mappings):
-                    ignored_subsequent_fields.add(marc_field.tag)
-            except Exception as ee:
-                logging.error(
-                    "map_field_according_to_mapping %s %s %s",
-                    marc_field.tag,
-                    marc_field.format_field(),
-                    json.dumps(mappings),
-                )
-                raise ee
-
-    def report_marc_stats(self, marc_field, bad_tags, legacy_ids, ignored_subsequent_fields):
-        self.migration_report.add_general_statistics("Total number of Tags processed")
-        self.report_bad_tags(marc_field, bad_tags, legacy_ids)
-        mapped = marc_field.tag in self.mappings
-        if marc_field.tag in ignored_subsequent_fields:
-            mapped = False
-        self.report_legacy_mapping(marc_field.tag, True, mapped)
-
-    def perform_proxy_mapping(self, marc_field):
-        proxy_mapping = next(iter(self.mappings.get("880", [])), [])
-        if proxy_mapping and "fieldReplacementRule" in proxy_mapping:
-            target_field = next(
-                (
-                    r["targetField"]
-                    for r in proxy_mapping["fieldReplacementRule"]
-                    if r["sourceDigits"] == marc_field["6"][:3]
-                ),
-                "",
-            )
-            mappings = self.mappings.get(target_field, {})
-
-            self.migration_report.add(
-                Blurbs.Field880Mappings,
-                f"Source digits: {marc_field['6'][:3]} Target field: {target_field}",
-            )
-        else:
-            raise TransformationProcessError("", "Mapping rules for 880 is missing. Halting")
-        return mappings
-
     def perform_additional_parsing(
         self,
         folio_instance: dict,
@@ -212,13 +147,18 @@ class BibsRulesMapper(RulesMapperBase):
         folio_instance["instanceFormatIds"] = list(
             set(self.get_instance_format_ids(marc_record, legacy_ids))
         )
-
         folio_instance["instanceTypeId"] = self.get_instance_type_id(marc_record, legacy_ids)
 
         folio_instance["modeOfIssuanceId"] = self.get_mode_of_issuance_id(marc_record, legacy_ids)
         self.handle_languages(folio_instance, marc_record, legacy_ids)
         self.handle_suppression(folio_instance, file_def)
         self.handle_holdings(marc_record)
+        if prec_titles := folio_instance.get("precedingTitles", []):
+            self.migration_report.add(Blurbs.PrecedingSuccedingTitles, f"{len(prec_titles)}")
+            del folio_instance["precedingTitles"]
+        if succ_titles := folio_instance.get("succeedingTitles", []):
+            del folio_instance["succeedingTitles"]
+            self.migration_report.add(Blurbs.PrecedingSuccedingTitles, f"{len(succ_titles)}")
 
     def handle_languages(self, folio_instance, marc_record, legacy_ids):
         if "languages" in folio_instance:
@@ -229,11 +169,13 @@ class BibsRulesMapper(RulesMapperBase):
             self.filter_langs(folio_instance["languages"], marc_record, legacy_ids)
         )
 
-    def handle_suppression(self, folio_instance, file_def):
-        folio_instance["discoverySuppress"] = file_def.suppressed
-        self.migration_report.add_general_statistics("Suppressed from discovery")
-        folio_instance["staffSuppress"] = file_def.staff_suppressed
-        self.migration_report.add_general_statistics("Staff suppressed")
+    @staticmethod
+    def get_instance_schema():
+        logging.info("Fetching Instance schema...")
+        instance_schema = FolioClient.get_latest_from_github(
+            "folio-org", "mod-inventory-storage", "ramls/instance.json"
+        )
+        return instance_schema
 
     def handle_holdings(self, marc_record: Record):
         if "852" in marc_record:
@@ -263,18 +205,8 @@ class BibsRulesMapper(RulesMapperBase):
 
     def wrap_up(self):
         logging.info("Mapper wrapping up")
-        self.store_hrid_settings()
-
-    def report_bad_tags(self, marc_field, bad_tags, legacy_ids):
-        if (
-            (not marc_field.tag.isnumeric())
-            and marc_field.tag != "LDR"
-            and marc_field.tag not in bad_tags
-        ):
-            self.migration_report.add(Blurbs.NonNumericTagsInRecord, marc_field.tag)
-            message = "Non-numeric tags in records"
-            Helper.log_data_issue(legacy_ids, message, marc_field.tag)
-            bad_tags.add(marc_field.tag)
+        if not self.task_configuration.never_update_hrid_settings:
+            self.hrid_handler.store_hrid_settings()
 
     def get_instance_type_id(self, marc_record, legacy_id):
         return_id = ""
@@ -284,7 +216,7 @@ class BibsRulesMapper(RulesMapperBase):
             match = next(
                 (
                     f["id"]
-                    for f in self.folio.instance_types
+                    for f in self.folio_client.instance_types
                     if f["name"].lower().replace(" ", "") == match_template
                 ),
                 "",
@@ -306,16 +238,11 @@ class BibsRulesMapper(RulesMapperBase):
                 )
             return match
 
-        if not self.folio.instance_types:
+        if not self.folio_client.instance_types:
             raise TransformationProcessError("", "No instance_types setup in tenant")
 
         if "336" in marc_record and "b" not in marc_record["336"]:
             self.migration_report.add(Blurbs.RecourceTypeMapping, "Subfield b not in 336")
-            Helper.log_data_issue(
-                legacy_id,
-                "Subfield b not in 336",
-                "",
-            )
             if "a" in marc_record["336"]:
                 return_id = get_folio_id_by_name(marc_record["336"]["a"])
 
@@ -323,7 +250,7 @@ class BibsRulesMapper(RulesMapperBase):
             f336_b = marc_record["336"]["b"].lower().replace(" ", "")
             f336_b_norm = f336_b.lower().replace(" ", "")
             t = self.conditions.get_ref_data_tuple_by_code(
-                self.folio.instance_types,
+                self.folio_client.instance_types,
                 "instance_types",
                 f336_b_norm,
             )
@@ -346,93 +273,112 @@ class BibsRulesMapper(RulesMapperBase):
 
         if not return_id:
             t = self.conditions.get_ref_data_tuple_by_code(
-                self.folio.instance_types, "instance_types", "zzz"
+                self.folio_client.instance_types, "instance_types", "zzz"
             )
             return_id = t[0]
         return return_id
 
+    def get_instance_format_id_by_code(self, legacy_id: str, code: str):
+        try:
+            match = next(f for f in self.folio_client.instance_formats if f["code"] == code)
+            self.migration_report.add(
+                Blurbs.InstanceFormat,
+                f"Successful match  - {code}->{match['name']}",
+            )
+            return match["id"]
+        except Exception:
+            # TODO: Distinguish between generated codes and proper 338bs
+            Helper.log_data_issue(legacy_id, "Instance format Code not found in FOLIO", code)
+            self.migration_report.add(
+                Blurbs.InstanceFormat,
+                f"Code '{code}' not found in FOLIO",
+            )
+            return ""
+
+    def get_instance_format_id_by_name(self, f337a: str, f338a: str, legacy_id: str):
+        f337a = f337a.lower().strip()
+        f338a = f338a.lower().strip()
+        match_template = f"{f337a} -- {f338a}"
+        try:
+            match = next(
+                f
+                for f in self.folio_client.instance_formats
+                if f["name"].lower() == match_template
+            )
+            self.migration_report.add(
+                Blurbs.InstanceFormat,
+                f"Successful matching on 337$a & 338$a - {match_template}->{match['name']}",
+            )
+            return match["id"]
+        except Exception:
+            Helper.log_data_issue(
+                legacy_id,
+                "Unsuccessful matching on 337$a and 338$a",
+                match_template,
+            )
+            self.migration_report.add(
+                Blurbs.InstanceFormat,
+                f"Unsuccessful matching on 337$a and 338$a - {match_template}",
+            )
+            return ""
+
+    def f338_source_is_rda_carrier(self, field: pymarc.Field):
+        if "2" not in field:
+            self.migration_report.add(
+                Blurbs.InstanceFormat,
+                ("Instance Format not mapped from field since 338$2 is missing"),
+            )
+            return False
+        elif field["2"].strip().startswith("rdacarrier"):
+            return True
+        self.migration_report.add(
+            Blurbs.InstanceFormat,
+            ("InstanceFormat not mapped since 338$2 (Source) " f"is set to {field['2']}. "),
+        )
+        return False
+
+    def get_instance_format_ids_from_a(
+        self, field_index, f_338: pymarc.Field, all_337s, legacy_id
+    ):
+        self.migration_report.add(
+            Blurbs.InstanceFormat,
+            "338$b is missing. Will try parse from 337$a and 338$a",
+        )
+        for a in f_338.get_subfields("a"):
+            corresponding_337 = all_337s[field_index] if field_index < len(all_337s) else None
+            if corresponding_337 and "a" in corresponding_337:
+                if fmt_id := self.get_instance_format_id_by_name(
+                    corresponding_337["a"], a, legacy_id
+                ):
+                    yield fmt_id
+
     def get_instance_format_ids(self, marc_record, legacy_id):
-        # Lambdas
-        def get_folio_id(code: str):
-            try:
-                match = next(f for f in self.folio.instance_formats if f["code"] == code)
-                self.migration_report.add(
-                    Blurbs.InstanceFormat,
-                    f"Successful match  - {code}->{match['name']}",
-                )
-                return match["id"]
-            except Exception:
-                # TODO: Distinguish between generated codes and proper 338bs
-                Helper.log_data_issue(legacy_id, "Instance format Code not found in FOLIO", code)
-                self.migration_report.add(
-                    Blurbs.InstanceFormat,
-                    f"Code '{code}' not found in FOLIO",
-                )
-                return ""
-
-        def get_folio_id_by_name(f337a: str, f338a: str, legacy_id: str):
-            f337a = f337a.lower().replace(" ", "")
-            f338a = f338a.lower().replace(" ", "")
-            match_template = f"{f337a} -- {f338a}"
-            try:
-                match = next(
-                    f
-                    for f in self.folio.instance_formats
-                    if f["name"].lower().replace(" ", "") == match_template
-                )
-                self.migration_report.add(
-                    Blurbs.InstanceFormat,
-                    f"Successful matching on 337$a & 338$a - {match_template}->{match['name']}",
-                )
-                return match["id"]
-            except Exception:
-                Helper.log_data_issue(
-                    legacy_id,
-                    "Unsuccessful matching on 337$a and 338$a",
-                    match_template,
-                )
-                self.migration_report.add(
-                    Blurbs.InstanceFormat,
-                    f"Unsuccessful matching on 337$a and 338$a - {match_template}",
-                )
-                return ""
-
         all_337s = marc_record.get_fields("337")
         all_338s = marc_record.get_fields("338")
-        for fidx, f in enumerate(all_338s):
-            source = f["2"] if "2" in f else "Not set"
-            if not source.strip().startswith("rdacarrier"):
-                self.migration_report.add(
-                    Blurbs.InstanceFormat,
-                    ("InstanceFormat not mapped since 338$2 (Source) " f"is set to {source}. "),
-                )
-            else:
-                if "b" not in f and "a" in f:
-                    self.migration_report.add(
-                        Blurbs.InstanceFormat,
-                        "338$b is missing. Will try parse from 337$a and 338$b",
+        for fidx, f_338 in enumerate(all_338s):
+            if self.f338_source_is_rda_carrier(f_338):
+                if "b" not in f_338 and "a" in f_338:
+                    yield from self.get_instance_format_ids_from_a(
+                        fidx, f_338, all_337s, legacy_id
                     )
-                    for a in f.get_subfields("a"):
-                        corresponding_337 = all_337s[fidx] if fidx < len(all_337s) else None
-                        if corresponding_337 and "a" in corresponding_337:
-                            fmt_id = get_folio_id_by_name(corresponding_337["a"], a, legacy_id)
-                            if fmt_id:
-                                yield fmt_id
 
-                for sfidx, b in enumerate(f.get_subfields("b")):
+                for sfidx, b in enumerate(f_338.get_subfields("b")):
                     b = b.replace(" ", "")
-                    if len(b) == 2:  # Normal 338b. should be able to map this
-                        yield get_folio_id(b)
+                    if len(b) == 2:
+                        # Normal 338b. should be able to map this
+                        yield self.get_instance_format_id_by_code(legacy_id, b)
                     elif len(b) == 1:
                         corresponding_337 = all_337s[fidx] if fidx < len(all_337s) else None
-                        if not corresponding_337:  # No matching 337. No use mapping the 338
+                        if not corresponding_337:
+                            # No matching 337. No use mapping the 338
                             s = "No corresponding 337 to 338 even though 338$b was one character"
                             Helper.log_data_issue(legacy_id, s, b)
                             self.migration_report.add(
                                 Blurbs.InstanceFormat,
                                 s,
                             )
-                        else:  # Corresponding 337. Try to combine the codes.
+                        else:
+                            # Corresponding 337. Try to combine the codes.
                             corresponding_b = (
                                 corresponding_337.get_subfields("b")[sfidx]
                                 if sfidx < len(corresponding_337.get_subfields("b"))
@@ -445,79 +391,9 @@ class BibsRulesMapper(RulesMapperBase):
                             else:
                                 combined_code = (corresponding_b + b).strip()
                                 if len(combined_code) == 2:
-                                    yield get_folio_id(combined_code)
-
-    def handle_hrid(self, folio_instance, marc_record: Record, legacy_ids) -> None:
-        """Create HRID if not mapped. Add hrid as MARC record 001
-
-        Args:
-            folio_instance (_type_): _description_
-            marc_record (Record): _description_
-            legacy_ids (_type_): _description_
-
-        Raises:
-            TransformationProcessError: _description_
-        """
-        if self.hrid_handling == HridHandling.default or "001" not in marc_record:
-            num_part = str(self.instance_hrid_counter).zfill(11)
-            folio_instance["hrid"] = f"{self.instance_hrid_prefix}{num_part}"
-            new_001 = Field(tag="001", data=folio_instance["hrid"])
-            try:
-                f_001 = marc_record["001"].value()
-                f_003 = marc_record["003"].value().strip() if "003" in marc_record else ""
-                self.migration_report.add(
-                    Blurbs.HridHandling, f'Values in 003: {f_003 or "Empty"}'
-                )
-
-                if self.task_configuration.deactivate035_from001:
-                    self.migration_report.add(
-                        Blurbs.HridHandling, "035 generation from 001 turned off"
-                    )
-                else:
-                    str_035 = f"({f_003}){f_001}" if f_003 else f"{f_001}"
-                    new_035 = Field(
-                        tag="035",
-                        indicators=[" ", " "],
-                        subfields=["a", str_035],
-                    )
-                    marc_record.add_ordered_field(new_035)
-                    self.migration_report.add(Blurbs.HridHandling, "Added 035 from 001")
-                marc_record.remove_fields("001")
-
-            except Exception:
-                if "001" in marc_record:
-                    s = "Failed to create 035 from 001"
-                    self.migration_report.add(Blurbs.HridHandling, s)
-                    Helper.log_data_issue(legacy_ids, s, marc_record["001"])
-                else:
-                    self.migration_report.add(
-                        Blurbs.HridHandling, "Legacy bib records without 001"
-                    )
-            marc_record.add_ordered_field(new_001)
-            self.migration_report.add(Blurbs.HridHandling, "Created HRID using default settings")
-            self.instance_hrid_counter += 1
-        elif self.hrid_handling == HridHandling.preserve001:
-            value = marc_record["001"].value()
-            if value in self.unique_001s:
-                self.migration_report.add(
-                    Blurbs.HridHandling, "Duplicate 001. Creating HRID instead"
-                )
-                Helper.log_data_issue(
-                    legacy_ids,
-                    "Duplicate 001 for record. HRID created for record",
-                    value,
-                )
-                num_part = str(self.instance_hrid_counter).zfill(11)
-                folio_instance["hrid"] = f"{self.instance_hrid_prefix}{num_part}"
-                new_001 = Field(tag="001", data=folio_instance["hrid"])
-                marc_record.add_ordered_field(new_001)
-                self.instance_hrid_counter += 1
-            else:
-                self.unique_001s.add(value)
-                folio_instance["hrid"] = value
-                self.migration_report.add(Blurbs.HridHandling, "Took HRID from 001")
-        else:
-            raise TransformationProcessError("", f"Unknown HRID handling: {self.hrid_handling}")
+                                    yield self.get_instance_format_id_by_code(
+                                        legacy_id, combined_code
+                                    )
 
     def get_mode_of_issuance_id(self, marc_record: Record, legacy_id: List[str]) -> str:
         level = marc_record.leader[7]
@@ -532,24 +408,27 @@ class BibsRulesMapper(RulesMapperBase):
             ret = next(
                 (
                     i["id"]
-                    for i in self.folio.modes_of_issuance
-                    if str(name).lower() == i["name"].lower()
+                    for i in self.folio_client.modes_of_issuance
+                    if name.lower() == i["name"].lower()
                 ),
                 "",
             )
+
             self.migration_report.add(Blurbs.MatchedModesOfIssuanceCode, f"{name} -- {ret}")
+
             if not ret:
                 self.migration_report.add(
-                    Blurbs.MatchedModesOfIssuanceCode,
-                    f"Unmatched level: {level}",
+                    Blurbs.MatchedModesOfIssuanceCode, f"Unmatched level: {level}"
                 )
+
                 return self.other_mode_of_issuance_id
             return ret
         except IndexError:
             self.migration_report.add(Blurbs.PossibleCleaningTasks, f"No Leader[7] in {legacy_id}")
+
             return self.other_mode_of_issuance_id
         except StopIteration as ee:
-            logging.exception(f"{marc_record.leader} {list(self.folio.modes_of_issuance)}")
+            logging.exception(f"{marc_record.leader} {list(self.folio_client.modes_of_issuance)}")
             raise ee from ee
 
     def get_nature_of_content(self, marc_record: Record) -> List[str]:
@@ -569,10 +448,8 @@ class BibsRulesMapper(RulesMapperBase):
         for lang_tag in lang_fields:
             if "2" in lang_tag:
                 self.migration_report.add(Blurbs.LanguageCodeSources, lang_tag["2"])
-                logging.info(
-                    "Field with other Language code\t%s\t%s",
-                    marc_record["001"],
-                    lang_tag.value(),
+                Helper.log_data_issue(
+                    legacy_id, "Field with other Language code", lang_tag.value()
                 )
             lang_codes = lang_tag.get_subfields(*list(subfields))
             for lang_code in lang_codes:
@@ -606,13 +483,15 @@ class BibsRulesMapper(RulesMapperBase):
         return list(languages)
 
     def fetch_language_codes(self) -> Generator[str, None, None]:
-        """fetches the list of standardized language codes from LoC
+        """Loads the  list of standardized language codes from LoC
 
         Yields:
             Generator[str, None, None]: _description_
         """
-        url = "https://www.loc.gov/standards/codelists/languages.xml"
-        tree = fromstring(requests.get(url).content)
+        path = Path(__file__).parent / "loc_language_codes.xml"
+        with open(path) as f:
+            lines = "".join(f.readlines())
+        tree = fromstring(lines)
         name_space = "{info:lc/xmlns/codelist-v1}"
         xpath_expr = "{0}languages/{0}language/{0}code".format(name_space)
         for code in tree.findall(xpath_expr):
@@ -645,47 +524,24 @@ class BibsRulesMapper(RulesMapperBase):
                     f"{m}: {language_value}",
                 )
 
-    def get_legacy_ids(
-        self, marc_record: Record, ils_flavour: IlsFlavour, index_or_legacy_id: str
-    ) -> List[str]:
+    def get_legacy_ids(self, marc_record: Record, idx: int) -> List[str]:
+        ils_flavour: IlsFlavour = self.task_configuration.ils_flavour
         if ils_flavour in {IlsFlavour.sierra, IlsFlavour.millennium}:
             return get_iii_bib_id(marc_record)
         elif ils_flavour == IlsFlavour.tag907y:
-            try:
-                return list(set(marc_record["907"].get_subfields("a", "y")))
-            except Exception as e:
-                raise TransformationRecordFailedError(
-                    index_or_legacy_id,
-                    (
-                        "907 $y and $a is missing is missing, although they is "
-                        "required for this legacy ILS choice"
-                    ),
-                    marc_record.as_json(),
-                ) from e
+            return RulesMapperBase.get_bib_id_from_907y(marc_record, idx)
         elif ils_flavour == IlsFlavour.tagf990a:
-            res = {f["a"].strip() for f in marc_record.get_fields("990") if "a" in f}
-            if marc_record["001"].format_field().strip():
-                res.add(marc_record["001"].format_field().strip())
-            if any(res):
-                self.migration_report.add_general_statistics("legacy id from 990$a")
-                return list(res)
+            return RulesMapperBase.get_bib_id_from_990a(marc_record, idx)
         elif ils_flavour == IlsFlavour.aleph:
             return self.get_aleph_bib_id(marc_record)
         elif ils_flavour in {IlsFlavour.voyager, "voyager", IlsFlavour.tag001}:
-            try:
-                return [marc_record["001"].format_field().strip()]
-            except Exception as e:
-                raise TransformationRecordFailedError(
-                    index_or_legacy_id,
-                    "001 is missing, although it is required for Voyager migrations",
-                    marc_record.as_json(),
-                ) from e
+            return RulesMapperBase.get_bib_id_from_001(marc_record, idx)
         elif ils_flavour == IlsFlavour.koha:
             try:
                 return [marc_record["999"]["c"]]
             except Exception as e:
                 raise TransformationRecordFailedError(
-                    index_or_legacy_id,
+                    idx,
                     "999 $c is missing, although it is required for this legacy ILS choice",
                     marc_record.as_json(),
                 ) from e
