@@ -5,14 +5,12 @@ import sys
 import time
 import traceback
 from datetime import datetime
-from datetime import timezone
 from typing import List
 from typing import Optional
 from uuid import uuid4
 
 import requests
 from folio_uuid.folio_namespaces import FOLIONamespaces
-from pydantic import BaseModel
 
 from folio_migration_tools.custom_exceptions import TransformationProcessError
 from folio_migration_tools.custom_exceptions import TransformationRecordFailedError
@@ -21,6 +19,7 @@ from folio_migration_tools.library_configuration import LibraryConfiguration
 from folio_migration_tools.migration_report import MigrationReport
 from folio_migration_tools.migration_tasks.migration_task_base import MigrationTaskBase
 from folio_migration_tools.report_blurbs import Blurbs
+from folio_migration_tools.task_configuration import AbstractTaskConfiguration
 
 
 def write_failed_batch_to_file(batch, file):
@@ -29,7 +28,24 @@ def write_failed_batch_to_file(batch, file):
 
 
 class BatchPoster(MigrationTaskBase):
-    class TaskConfiguration(BaseModel):
+    """Batchposter
+
+    Args:
+        MigrationTaskBase (_type_): _description_
+
+    Raises:
+        ee: _description_
+        TransformationRecordFailedError: _description_
+        TransformationProcessError: _description_
+        TransformationRecordFailedError: _description_
+        TransformationRecordFailedError: _description_
+        TransformationProcessError: _description_
+
+    Returns:
+        _type_: _description_
+    """
+
+    class TaskConfiguration(AbstractTaskConfiguration):
         name: str
         migration_task_type: str
         object_type: str
@@ -48,16 +64,14 @@ class BatchPoster(MigrationTaskBase):
         use_logging: bool = True,
     ):
         super().__init__(library_config, task_config, use_logging)
-        self.task_config = task_config
-        self.start_datetime = datetime.now(timezone.utc)
         self.migration_report = MigrationReport()
         self.performing_rerun = False
         self.failed_ids = []
         self.first_batch = True
-        self.api_path = list_objects(self.task_config.object_type)
+        self.api_path = list_objects(self.task_configuration.object_type)
         self.snapshot_id = str(uuid4())
         self.failed_objects = []
-        self.batch_size = self.task_config.batch_size
+        self.batch_size = self.task_configuration.batch_size
         logging.info("Batch size is %s", self.batch_size)
         self.processed = 0
         self.failed_batches = 0
@@ -71,10 +85,10 @@ class BatchPoster(MigrationTaskBase):
     def do_work(self):
         try:
             batch = []
-            if self.task_config.object_type == "SRS":
+            if self.task_configuration.object_type == "SRS":
                 self.create_snapshot()
             with open(self.folder_structure.failed_recs_path, "w") as failed_recs_file:
-                for file_def in self.task_config.files:
+                for file_def in self.task_configuration.files:
                     path = self.folder_structure.results_folder / file_def.file_name
                     with open(path) as rows:
                         logging.info("Running %s", path)
@@ -83,11 +97,17 @@ class BatchPoster(MigrationTaskBase):
                             last_row = row
                             if row.strip():
                                 try:
-                                    if self.task_config.object_type == "Extradata":
+                                    if self.task_configuration.object_type == "Extradata":
                                         self.post_extra_data(row, self.processed, failed_recs_file)
+                                    elif not list_objects(self.task_configuration.object_type)[
+                                        "is_batch"
+                                    ]:
+                                        self.post_single_records(
+                                            row, self.processed, failed_recs_file
+                                        )
                                     else:
                                         json_rec = json.loads(row.split("\t")[-1])
-                                        if self.task_config.object_type == "SRS":
+                                        if self.task_configuration.object_type == "SRS":
                                             json_rec["snapshotId"] = self.snapshot_id
                                         if self.processed == 1:
                                             logging.info(json.dumps(json_rec, indent=True))
@@ -118,7 +138,7 @@ class BatchPoster(MigrationTaskBase):
                                     )
                                     batch = []
 
-                if self.task_config.object_type != "Extradata" and any(batch):
+                if self.task_configuration.object_type != "Extradata" and any(batch):
                     try:
                         self.post_batch(batch, failed_recs_file, self.processed)
                     except Exception as exception:
@@ -127,7 +147,7 @@ class BatchPoster(MigrationTaskBase):
                         )
                 logging.info("Done posting %s records. ", (self.processed))
         except Exception as ee:
-            if self.task_config.object_type == "SRS":
+            if self.task_configuration.object_type == "SRS":
                 self.commit_snapshot()
             raise ee
 
@@ -156,6 +176,32 @@ class BatchPoster(MigrationTaskBase):
                 self.num_failures,
             )
 
+    def post_single_records(self, row: str, num_records: int, failed_recs_file):
+        kind = list_objects(self.task_configuration.object_type)
+        if kind["is_batch"]:
+            raise TypeError("This record type supports batch processing, use post_batch method")
+        api_endpoint = kind.get("api_endpoint")
+        url = f"{self.folio_client.okapi_url}{api_endpoint}"
+        response = self.post_objects(url, row)
+        if response.status_code == 201:
+            self.num_posted += 1
+        elif response.status_code == 422:
+            self.num_failures += 1
+            error_msg = json.loads(response.text)["errors"][0]["message"]
+            logging.error("Row %s\tHTTP %s\t %s", num_records, response.status_code, error_msg)
+            if "id value already exists" not in json.loads(response.text)["errors"][0]["message"]:
+                failed_recs_file.write(row)
+        else:
+            self.num_failures += 1
+            logging.error("Row %s\tHTTP %s\t%s", num_records, response.status_code, response.text)
+            failed_recs_file.write(row)
+        if num_records % 50 == 0:
+            logging.info(
+                "%s records posted successfully. %s failed",
+                self.num_posted,
+                self.num_failures,
+            )
+
     def post_objects(self, url, body):
         return requests.post(
             url, headers=self.folio_client.okapi_headers, data=body.encode("utf-8")
@@ -184,7 +230,7 @@ class BatchPoster(MigrationTaskBase):
         )
         logging.info(
             "Failing row, either the one shown here or the next row in %s",
-            self.task_config.file.file_name,
+            self.task_configuration.file.file_name,
         )
         logging.info(last_row)
         logging.info("=========Stack trace==============")
@@ -256,7 +302,7 @@ class BatchPoster(MigrationTaskBase):
             # Likely a json parsing error
             logging.error(response.text)
             raise TransformationProcessError("", "HTTP 400. Somehting is wrong. Quitting")
-        elif self.task_config.object_type == "SRS" and response.status_code == 500:
+        elif self.task_configuration.object_type == "SRS" and response.status_code >= 500:
             logging.info(
                 "Post failed. Size: %s Waiting 30s until reposting. Number of tries: %s of 5",
                 get_req_size(response),
@@ -278,6 +324,8 @@ class BatchPoster(MigrationTaskBase):
             try:
                 logging.info(response.text)
                 resp = json.dumps(response, indent=4)
+            except TypeError:
+                resp = response
             except Exception:
                 logging.exception("something unexpected happened")
                 resp = response
@@ -290,7 +338,7 @@ class BatchPoster(MigrationTaskBase):
             )
 
     def do_post(self, batch):
-        kind = list_objects(self.task_config.object_type)
+        kind = list_objects(self.task_configuration.object_type)
         path = kind["api_endpoint"]
         url = self.folio_client.okapi_url + path
         if kind["object_name"] == "users":
@@ -305,9 +353,10 @@ class BatchPoster(MigrationTaskBase):
 
     def wrap_up(self):
         logging.info("Done. Wrapping up")
-        if self.task_config.object_type == "SRS":
+        self.extradata_writer.flush()
+        if self.task_configuration.object_type == "SRS":
             self.commit_snapshot()
-        if self.task_config.object_type != "Extradata":
+        if self.task_configuration.object_type != "Extradata":
             logging.info(
                 (
                     "Failed records: %s failed records in %s "
@@ -318,7 +367,7 @@ class BatchPoster(MigrationTaskBase):
                 self.folder_structure.failed_recs_path,
             )
         else:
-            logging.info("Done posting % records. % failed", self.num_posted, self.num_failures)
+            logging.info("Done posting %s records. %s failed", self.num_posted, self.num_failures)
 
         run = "second time" if self.performing_rerun else "first time"
         self.migration_report.set(
@@ -332,25 +381,28 @@ class BatchPoster(MigrationTaskBase):
         )
         self.rerun_run()
         with open(self.folder_structure.migration_reports_file, "w+") as report_file:
-            report_file.write("# Reserves migration results   \n")
-            report_file.write(f"Time Finished: {datetime.isoformat(datetime.now(timezone.utc))}\n")
-            self.migration_report.write_migration_report(report_file, self.start_datetime)
+            self.migration_report.write_migration_report(
+                f"{self.task_configuration.object_type} loading report",
+                report_file,
+                self.start_datetime,
+            )
+        self.clean_out_empty_logs()
 
     def rerun_run(self):
-        if self.task_config.rerun_failed_records and (self.num_failures > 0):
+        if self.task_configuration.rerun_failed_records and (self.num_failures > 0):
             logging.info(
                 "Rerunning the %s failed records from the load with a batchsize of 1",
                 self.num_failures,
             )
             try:
-                self.task_config.batch_size = 1
-                self.task_config.files = [
+                self.task_configuration.batch_size = 1
+                self.task_configuration.files = [
                     FileDefinition(file_name=str(self.folder_structure.failed_recs_path.name))
                 ]
                 temp_report = copy.deepcopy(self.migration_report)
                 temp_start = self.start_datetime
-                self.task_config.rerun_failed_records = False
-                self.__init__(self.task_config, self.library_configuration)
+                self.task_configuration.rerun_failed_records = False
+                self.__init__(self.task_configuration, self.library_configuration)
                 self.performing_rerun = True
                 self.migration_report = temp_report
                 self.start_datetime = temp_start
@@ -418,33 +470,52 @@ def list_objects(object_type: str):
         "Items": {
             "object_name": "items",
             "api_endpoint": "/item-storage/batch/synchronous?upsert=true",
+            "is_batch": True,
             "total_records": False,
             "addSnapshotId": False,
         },
         "Holdings": {
             "object_name": "holdingsRecords",
             "api_endpoint": "/holdings-storage/batch/synchronous?upsert=true",
+            "is_batch": True,
             "total_records": False,
             "addSnapshotId": False,
         },
         "Instances": {
             "object_name": "instances",
             "api_endpoint": "/instance-storage/batch/synchronous?upsert=true",
+            "is_batch": True,
+            "total_records": False,
+            "addSnapshotId": False,
+        },
+        "Authorities": {
+            "object_name": "",
+            "api_endpoint": "/authority-storage/authorities",
+            "is_batch": False,
             "total_records": False,
             "addSnapshotId": False,
         },
         "SRS": {
             "object_name": "records",
             "api_endpoint": "/source-storage/batch/records",
+            "is_batch": True,
             "total_records": True,
             "addSnapshotId": True,
         },
         "Users": {
             "object_name": "users",
             "api_endpoint": "/user-import",
+            "is_batch": True,
             "total_records": True,
             "addSnapshotId": False,
         },
+        "Organizations": {
+            "object_name": "",
+            "api_endpoint": "/organizations/organizations",
+            "is_batch": False,
+            "total_records": False,
+            "addSnapshotId": False,
+        },
     }
 
     try:
@@ -479,6 +550,7 @@ def get_extradata_endpoint(object_name: str, string_object: str):
         "notes": "notes",
         "course": "coursereserves/courses",
         "courselisting": "coursereserves/courselistings",
+        "contacts": "organizations-storage/contacts",
     }
     if object_name == "instructor":
         instructor = json.loads(string_object)
