@@ -14,7 +14,6 @@ from typing import Optional
 from folio_uuid import FolioUUID
 from folio_uuid.folio_namespaces import FOLIONamespaces
 from folioclient import FolioClient
-from pydantic.main import BaseModel
 from requests import HTTPError
 
 from folio_migration_tools.custom_exceptions import TransformationProcessError
@@ -30,14 +29,16 @@ from folio_migration_tools.mapping_file_transformation.holdings_mapper import (
 from folio_migration_tools.mapping_file_transformation.mapping_file_mapper_base import (
     MappingFileMapperBase,
 )
+from folio_migration_tools.marc_rules_transformation.hrid_handler import HRIDHandler
 from folio_migration_tools.migration_tasks.migration_task_base import MigrationTaskBase
+from folio_migration_tools.task_configuration import AbstractTaskConfiguration
 
 csv.field_size_limit(int(ctypes.c_ulong(-1).value // 2))
 csv.register_dialect("tsv", delimiter="\t")
 
 
 class HoldingsCsvTransformer(MigrationTaskBase):
-    class TaskConfiguration(BaseModel):
+    class TaskConfiguration(AbstractTaskConfiguration):
         name: str
         migration_task_type: str
         hrid_handling: HridHandling
@@ -54,6 +55,8 @@ class HoldingsCsvTransformer(MigrationTaskBase):
             "permanentLocationId",
             "callNumber",
         ]
+        reset_hrid_settings: Optional[bool] = False
+        never_update_hrid_settings: Optional[bool] = False
 
     @staticmethod
     def get_object_type() -> FOLIONamespaces:
@@ -123,6 +126,16 @@ class HoldingsCsvTransformer(MigrationTaskBase):
 
             else:
                 logging.info("No file of legacy holdings setup.")
+
+            if (
+                self.task_configuration.reset_hrid_settings
+                and not self.task_configuration.never_update_hrid_settings
+            ):
+                hrid_handler = HRIDHandler(
+                    self.folio_client, HridHandling.default, self.mapper.migration_report, True
+                )
+                hrid_handler.reset_holdings_hrid_counter()
+
         except HTTPError as http_error:
             logging.critical(http_error)
             sys.exit(1)
@@ -130,6 +143,8 @@ class HoldingsCsvTransformer(MigrationTaskBase):
             logging.critical(process_error)
             logging.critical("Halting.")
             sys.exit(1)
+        except json.JSONDecodeError as jde:
+            raise jde
         except Exception as exception:
             logging.info("\n=======ERROR===========")
             logging.info(exception)
@@ -182,7 +197,7 @@ class HoldingsCsvTransformer(MigrationTaskBase):
         with open(self.folder_structure.instance_id_map_path, "r") as instance_id_map_file:
             for index, json_string in enumerate(instance_id_map_file):
                 # Format:{"legacy_id", "folio_id","instanceLevelCallNumber"}
-                if index % 100000 == 0:
+                if index % 500000 == 0:
                     print(f"{index} instance ids loaded to map", end="\r")
                 map_object = json.loads(json_string)
                 res[map_object["legacy_id"]] = map_object
@@ -198,7 +213,7 @@ class HoldingsCsvTransformer(MigrationTaskBase):
             except Exception as ee:
                 error_str = (
                     f"Processing of {file_def.file_name} failed:\n{ee}."
-                    "\nCheck source files for empty lines or missing reference data"
+                    "\nCheck source files for empty rows or missing reference data"
                 )
                 logging.critical(error_str)
                 print(f"\n{error_str}\nHalting")
@@ -208,7 +223,8 @@ class HoldingsCsvTransformer(MigrationTaskBase):
         )
 
     def wrap_up(self):
-        logging.info("Work done. Wrapping up...")
+        logging.info("Done. Transformer wrapping up...")
+        self.extradata_writer.flush()
         if any(self.holdings):
             logging.info(
                 "Saving holdings created to %s",
@@ -220,8 +236,8 @@ class HoldingsCsvTransformer(MigrationTaskBase):
                         # Prevent the first item in a boundwith to be overwritten
                         # TODO: Find out why not
                         # if legacy_id not in self.holdings_id_map:
-                        self.holdings_id_map[legacy_id] = self.mapper.get_id_map_dict(
-                            legacy_id, holding
+                        self.holdings_id_map[legacy_id] = self.mapper.get_id_map_tuple(
+                            legacy_id, holding, self.object_type
                         )
                     Helper.write_to_file(holdings_file, holding)
                     self.mapper.migration_report.add_general_statistics(
@@ -231,12 +247,10 @@ class HoldingsCsvTransformer(MigrationTaskBase):
                 self.folder_structure.holdings_id_map_path, self.holdings_id_map
             )
         with open(self.folder_structure.migration_reports_file, "w") as migration_report_file:
-            logging.info(
-                "Writing migration- and mapping report to %s",
-                self.folder_structure.migration_reports_file,
-            )
             self.mapper.migration_report.write_migration_report(
-                migration_report_file, self.mapper.start_datetime
+                "Holdings transformation report",
+                migration_report_file,
+                self.mapper.start_datetime,
             )
             Helper.print_mapping_report(
                 migration_report_file,
@@ -245,6 +259,7 @@ class HoldingsCsvTransformer(MigrationTaskBase):
                 self.mapper.mapped_legacy_fields,
             )
         logging.info("All done!")
+        self.clean_out_empty_logs()
 
     def validate_merge_criterias(self):
         holdings_schema = self.folio_client.get_holdings_schema()
@@ -373,9 +388,8 @@ class HoldingsCsvTransformer(MigrationTaskBase):
             self.mapper.migration_report.add_general_statistics("Bound-with holdings created")
             yield bound_with_holding
 
-    @staticmethod
     def generate_boundwith_part(
-        folio_client: FolioClient, legacy_item_id: str, bound_with_holding: dict
+        self, folio_client: FolioClient, legacy_item_id: str, bound_with_holding: dict
     ):
         part = {
             "id": str(uuid.uuid4()),
@@ -388,7 +402,7 @@ class HoldingsCsvTransformer(MigrationTaskBase):
                 )
             ),
         }
-        logging.log(25, f"boundwithPart\t{json.dumps(part)}")
+        self.mapper.extradata_writer.write("boundwithPart", part)
 
     def merge_holding_in(
         self, incoming_holding: dict, instance_ids: list[str], legacy_item_id: str
@@ -417,8 +431,8 @@ class HoldingsCsvTransformer(MigrationTaskBase):
                 self.generate_boundwith_part(
                     self.folio_client, legacy_item_id, self.holdings[bw_key]
                 )
-                self.holdings_id_map[legacy_item_id] = self.mapper.get_id_map_dict(
-                    legacy_item_id, self.holdings[bw_key]
+                self.holdings_id_map[legacy_item_id] = self.mapper.get_id_map_tuple(
+                    legacy_item_id, self.holdings[bw_key], self.object_type
                 )
                 self.mapper.migration_report.add_general_statistics(
                     "BW Items found tied to previously created BW Holdings"
@@ -460,8 +474,3 @@ class HoldingsCsvTransformer(MigrationTaskBase):
             raise TransformationProcessError("", "No holdings source with name MARC in tenant")
         logging.info(json.dumps(res, indent=4))
         return res
-
-
-def dedupe(list_of_dicts):
-    # TODO: Move to interface or parent class
-    return [dict(t) for t in {tuple(d.items()) for d in list_of_dicts}]
