@@ -7,23 +7,25 @@ import time
 import traceback
 from datetime import datetime
 from datetime import timedelta
-from datetime import timezone
 from typing import Optional
 from urllib.error import HTTPError
+from zoneinfo import ZoneInfo
 
 import requests
 from dateutil import parser as du_parser
 from folio_uuid.folio_namespaces import FOLIONamespaces
-from pydantic import BaseModel
 
 from folio_migration_tools.circulation_helper import CirculationHelper
-from folio_migration_tools.custom_dict import InsensitiveDictReader
 from folio_migration_tools.helper import Helper
 from folio_migration_tools.library_configuration import FileDefinition
 from folio_migration_tools.library_configuration import LibraryConfiguration
+from folio_migration_tools.mapping_file_transformation.mapping_file_mapper_base import (
+    MappingFileMapperBase,
+)
 from folio_migration_tools.migration_report import MigrationReport
 from folio_migration_tools.migration_tasks.migration_task_base import MigrationTaskBase
 from folio_migration_tools.report_blurbs import Blurbs
+from folio_migration_tools.task_configuration import AbstractTaskConfiguration
 from folio_migration_tools.transaction_migration.legacy_loan import LegacyLoan
 from folio_migration_tools.transaction_migration.transaction_result import (
     TransactionResult,
@@ -31,11 +33,10 @@ from folio_migration_tools.transaction_migration.transaction_result import (
 
 
 class LoansMigrator(MigrationTaskBase):
-    class TaskConfiguration(BaseModel):
+    class TaskConfiguration(AbstractTaskConfiguration):
         name: str
-        utc_difference: int
         migration_task_type: str
-        open_loans_file: FileDefinition
+        open_loans_files: list[FileDefinition]
         fallback_service_point_id: str
         starting_row: Optional[int] = 1
         item_files: Optional[list[FileDefinition]] = []
@@ -51,7 +52,13 @@ class LoansMigrator(MigrationTaskBase):
         library_config: LibraryConfiguration,
     ):
         csv.register_dialect("tsv", delimiter="\t")
-        self.start_datetime = datetime.now(timezone.utc)
+        self.patron_item_combos: set = set()
+        self.t0 = time.time()
+        self.num_duplicate_loans = 0
+        self.skipped_since_already_added = 0
+        self.processed_items: set = set()
+        self.failed: dict = {}
+        self.failed_and_not_dupe: dict = {}
         self.migration_report = MigrationReport()
         self.valid_legacy_loans = []
         super().__init__(library_config, task_configuration)
@@ -60,21 +67,52 @@ class LoansMigrator(MigrationTaskBase):
             task_configuration.fallback_service_point_id,
             self.migration_report,
         )
-        with open(
-            self.folder_structure.legacy_records_folder
-            / task_configuration.open_loans_file.file_name,
-            "r",
-            encoding="utf-8",
-        ) as loans_file:
-            self.semi_valid_legacy_loans = list(
-                self.load_and_validate_legacy_loans(
-                    InsensitiveDictReader(loans_file, dialect="tsv")
+        logging.info("Check that SMTP is disabled before migrating loans")
+        self.check_smtp_config()
+        logging.info("Proceeding with loans migration")
+        logging.info("Attempting to retrieve tenant timezone configuration...")
+        my_path = "/configurations/entries?query=(module==ORG%20and%20configName==localeSettings)"
+        try:
+            self.tenant_timezone_str = json.loads(
+                self.folio_client.folio_get_single_object(my_path)["configs"][0]["value"]
+            )["timezone"]
+            logging.info("Tenant timezone is: %s", self.tenant_timezone_str)
+        except Exception:
+            logging.info('Tenant locale settings not available. Using "UTC".')
+            self.tenant_timezone_str = "UTC"
+        self.tenant_timezone = ZoneInfo(self.tenant_timezone_str)
+        self.semi_valid_legacy_loans = []
+        for file_def in task_configuration.open_loans_files:
+            loans_file_path = self.folder_structure.legacy_records_folder / file_def.file_name
+            with open(loans_file_path, "r", encoding="utf-8") as loans_file:
+                total_rows, empty_rows, reader = MappingFileMapperBase._get_delimited_file_reader(
+                    loans_file, loans_file_path
                 )
-            )
-            logging.info(
-                "Loaded and validated %s loans in file",
-                len(self.semi_valid_legacy_loans),
-            )
+                logging.info("Source data file contains %d rows", total_rows)
+                logging.info("Source data file contains %d empty rows", empty_rows)
+                self.migration_report.set(
+                    Blurbs.GeneralStatistics,
+                    f"Total rows in {loans_file_path.name}",
+                    total_rows,
+                )
+                self.migration_report.set(
+                    Blurbs.GeneralStatistics,
+                    f"Empty rows in {loans_file_path.name}",
+                    empty_rows,
+                )
+                self.semi_valid_legacy_loans.extend(
+                    self.load_and_validate_legacy_loans(
+                        reader,
+                        file_def.service_point_id or task_configuration.fallback_service_point_id,
+                    )
+                )
+
+                logging.info(
+                    "Loaded and validated %s loans in file from %s",
+                    len(self.semi_valid_legacy_loans),
+                    file_def.file_name,
+                )
+        logging.info("Loaded and validated %s loans in total", len(self.semi_valid_legacy_loans))
         if any(self.task_configuration.item_files) or any(self.task_configuration.patron_files):
             self.valid_legacy_loans = list(self.check_barcodes())
             logging.info(
@@ -87,16 +125,32 @@ class LoansMigrator(MigrationTaskBase):
                 "previously migrated objects"
             )
             self.valid_legacy_loans = self.semi_valid_legacy_loans
-        self.patron_item_combos = set()
-        self.t0 = time.time()
-        self.num_duplicate_loans = 0
-        self.skipped_since_already_added = 0
-        self.processed_items = set()
-        self.failed = {}
-        self.failed_and_not_dupe = {}
         logging.info("Starting row number is %s", task_configuration.starting_row)
         logging.info("Init completed")
 
+    def check_smtp_config(self):
+        okapi_config_base_path = "/configurations/entries"
+        okapi_config_query = "(module=={}%20and%20configName=={}%20and%20code=={})"
+        okapi_config_limit = 1000
+        okapi_config_module = "SMTP_SERVER"
+        okapi_config_name = "smtp"
+        okapi_config_code = "EMAIL_SMTP_HOST_DISABLED"
+        smtp_config_path = (
+            okapi_config_base_path
+            + "?"
+            + str(okapi_config_limit)
+            + "&query="
+            + okapi_config_query.format(okapi_config_module, okapi_config_name, okapi_config_code)
+        )
+        print_smtp_warning()
+        if not self.folio_client.folio_get_single_object(smtp_config_path)["configs"]:
+            logging.warn("SMTP connection not disabled...")
+            for i in range(10, 0, -1):
+                sys.stdout.write("Pausing for {:02d} seconds. Press Ctrl+C to exit...\r".format(i))
+                time.sleep(1)
+        else:
+            logging.info("SMTP connection is disabled...")
+
     def do_work(self):
         logging.info("Starting")
         starting_index = (
@@ -108,7 +162,7 @@ class LoansMigrator(MigrationTaskBase):
             logging.info(f"Skipping {(starting_index)} records")
         for num_loans, legacy_loan in enumerate(self.valid_legacy_loans[starting_index:], start=1):
             t0_migration = time.time()
-            self.migration_report.add_general_statistics("Processed loans")
+            self.migration_report.add_general_statistics("Processed pre-validated loans")
             try:
                 self.checkout_single_loan(legacy_loan)
             except Exception as ee:
@@ -129,17 +183,23 @@ class LoansMigrator(MigrationTaskBase):
 
         if res_checkout.was_successful:
             self.migration_report.add(Blurbs.Details, "Checked out on first try")
+            self.migration_report.add_general_statistics("Successfully checked out")
             self.set_renewal_count(legacy_loan, res_checkout)
             self.set_new_status(legacy_loan, res_checkout)
         elif res_checkout.should_be_retried:
             res_checkout2 = self.handle_checkout_failure(legacy_loan, res_checkout)
             if res_checkout2.was_successful and res_checkout2.folio_loan:
                 self.migration_report.add(Blurbs.Details, "Checked out on second try")
+                self.migration_report.add_general_statistics("Successfully checked out")
                 logging.info("Checked out on second try")
                 self.set_renewal_count(legacy_loan, res_checkout2)
                 self.set_new_status(legacy_loan, res_checkout2)
             elif legacy_loan.item_barcode not in self.failed:
                 self.failed[legacy_loan.item_barcode] = legacy_loan
+                self.migration_report.add_general_statistics("Failed loans")
+                Helper.log_data_issue(
+                    "", "Loans failing during checkout", json.dumps(legacy_loan.to_dict())
+                )
                 logging.error("Failed on second try: %s", res_checkout2.error_message)
                 self.migration_report.add(
                     Blurbs.Details,
@@ -147,10 +207,15 @@ class LoansMigrator(MigrationTaskBase):
                 )
         elif not res_checkout.should_be_retried:
             logging.error("Failed first time. No retries: %s", res_checkout.error_message)
+            self.migration_report.add_general_statistics("Failed loans")
             self.migration_report.add(
                 Blurbs.Details,
                 f"Failed 1st time. No retries: {res_checkout.migration_report_message}",
             )
+            self.failed[legacy_loan.item_barcode] = legacy_loan
+            Helper.log_data_issue(
+                "", "Loans failing during checkout", json.dumps(legacy_loan.to_dict())
+            )
 
     def set_new_status(self, legacy_loan: LegacyLoan, res_checkout: TransactionResult):
         """Updates checkout loans with their destination statuses
@@ -175,16 +240,15 @@ class LoansMigrator(MigrationTaskBase):
     def wrap_up(self):
         for k, v in self.failed.items():
             self.failed_and_not_dupe[k] = [v.to_dict()]
-        self.migration_report.set(
-            Blurbs.GeneralStatistics, "Failed loans", len(self.failed_and_not_dupe)
-        )
+        print(f"Wrapping up. Unique loans in failed:{len(self.failed_and_not_dupe)}")
 
         self.write_failed_loans_to_file()
 
         with open(self.folder_structure.migration_reports_file, "w+") as report_file:
-            report_file.write("# Loans migration results   \n")
-            report_file.write(f"Time Finished: {datetime.isoformat(datetime.now(timezone.utc))}\n")
-            self.migration_report.write_migration_report(report_file, self.start_datetime)
+            self.migration_report.write_migration_report(
+                "Loans migration report", report_file, self.start_datetime
+            )
+        self.clean_out_empty_logs()
 
     def write_failed_loans_to_file(self):
         csv_columns = [
@@ -219,6 +283,9 @@ class LoansMigrator(MigrationTaskBase):
                 )
                 yield loan
             else:
+                # Add this loan to failed loans for later correction and re-run.
+                self.failed[loan.item_barcode] = loan
+                self.migration_report.add_general_statistics("Failed loans")
                 self.migration_report.add(
                     Blurbs.DiscardedLoans,
                     f"Loans discarded. Had migrated item barcode: {has_item_barcode}. "
@@ -235,36 +302,46 @@ class LoansMigrator(MigrationTaskBase):
                     json.dumps(loan.to_dict()),
                 )
 
-    def load_and_validate_legacy_loans(self, loans_reader):
+    def load_and_validate_legacy_loans(self, loans_reader, service_point_id: str) -> list:
+        results = []
         num_bad = 0
         logging.info("Validating legacy loans in file...")
         for legacy_loan_count, legacy_loan_dict in enumerate(loans_reader):
             try:
                 legacy_loan = LegacyLoan(
                     legacy_loan_dict,
-                    self.task_configuration.utc_difference,
+                    service_point_id,
+                    self.migration_report,
+                    self.tenant_timezone,
                     legacy_loan_count,
                 )
                 if any(legacy_loan.errors):
                     num_bad += 1
-                    self.migration_report.add_general_statistics("Discarded Loans")
+                    self.migration_report.add_general_statistics("Loans failed pre-validation")
+                    self.migration_report.add_general_statistics("Failed loans")
                     for error in legacy_loan.errors:
                         self.migration_report.add(
                             Blurbs.DiscardedLoans, f"{error[0]} - {error[1]}"
                         )
+                    # Add this loan to failed loans for later correction and re-run.
+                    self.failed[
+                        legacy_loan.item_barcode or f"no_barcode_{legacy_loan_count}"
+                    ] = legacy_loan
                 else:
-                    yield legacy_loan
+                    results.append(legacy_loan)
             except ValueError as ve:
                 logging.exception(ve)
         logging.info(
-            f"Done validating {legacy_loan_count} legacy loans with {num_bad} rotten apples"
+            f"Done validating {legacy_loan_count + 1} legacy loans out of which "
+            f"{num_bad} where discarded."
         )
-        if num_bad / legacy_loan_count > 0.5:
-            q = num_bad / legacy_loan_count
+        if num_bad / (legacy_loan_count + 1) > 0.5:
+            q = num_bad / (legacy_loan_count + 1)
             logging.error("%s percent of loans failed to validate.", (q * 100))
             self.migration_report.log_me()
             logging.critical("Halting...")
             sys.exit(1)
+        return results
 
     def handle_checkout_failure(
         self, legacy_loan, folio_checkout: TransactionResult
@@ -307,7 +384,7 @@ class LoansMigrator(MigrationTaskBase):
             if legacy_loan.item_barcode not in self.failed:
                 self.failed[legacy_loan.item_barcode] = legacy_loan
             else:
-                logging.debug(
+                logging.info(
                     f"Loan already in failed. item barcode {legacy_loan.item_barcode} "
                     f"Patron barcode: {legacy_loan.patron_barcode}"
                 )
@@ -353,7 +430,6 @@ class LoansMigrator(MigrationTaskBase):
         due_date = du_parser.isoparse(str(legacy_loan.due_date))
         out_date = du_parser.isoparse(str(legacy_loan.out_date))
         renewal_count = legacy_loan.renewal_count
-        # TODO: add logging instead of print out
         try:
             loan_to_put = copy.deepcopy(folio_loan)
             del loan_to_put["metadata"]
@@ -416,7 +492,6 @@ class LoansMigrator(MigrationTaskBase):
         else:
             logging.error(f"Unsuccessfully declared loan {folio_loan} as lost")
             self.migration_report.add(Blurbs.Details, "Unsuccessfully declared loan as lost")
-        # TODO: Exception handling
 
     def claim_returned(self, folio_loan):
         claim_returned_url = f"/circulation/loans/{folio_loan['id']}/claim-item-returned"
@@ -437,7 +512,6 @@ class LoansMigrator(MigrationTaskBase):
                 Blurbs.Details,
                 f"Unsuccessfully declared loan {folio_loan} as Claimed returned",
             )
-        # TODO: Exception handling
 
     def set_item_status(self, legacy_loan: LegacyLoan):
         try:
@@ -596,3 +670,14 @@ def timings(t0, t0func, num_objects):
         f"Total objects: {num_objects}\tTotal elapsed: {elapsed:.2f}\t"
         f"Average per object: {avg:.2f}\tElapsed this time: {elapsed_func:.2f}"
     )
+
+
+def print_smtp_warning():
+    s = """
+          _____   __  __   _____   ______   ___
+         / ____| |  \/  | |_   _| |  __  | |__ \\
+        | (___   | \  / |   | |   | |__|_|    ) |
+         \___ \  | |\/| |   | |   | |        / /
+        |_____/  |_|  |_|   |_|   |_|       (_)
+    """  # noqa: E501, W605
+    print(s)
