@@ -3,12 +3,10 @@ import json
 import logging
 import sys
 import time
-from datetime import datetime
-from datetime import timezone
 from typing import Optional
+from zoneinfo import ZoneInfo
 
 from folio_uuid.folio_namespaces import FOLIONamespaces
-from pydantic import BaseModel
 
 from folio_migration_tools.circulation_helper import CirculationHelper
 from folio_migration_tools.custom_dict import InsensitiveDictReader
@@ -18,13 +16,13 @@ from folio_migration_tools.library_configuration import LibraryConfiguration
 from folio_migration_tools.migration_report import MigrationReport
 from folio_migration_tools.migration_tasks.migration_task_base import MigrationTaskBase
 from folio_migration_tools.report_blurbs import Blurbs
+from folio_migration_tools.task_configuration import AbstractTaskConfiguration
 from folio_migration_tools.transaction_migration.legacy_request import LegacyRequest
 
 
 class RequestsMigrator(MigrationTaskBase):
-    class TaskConfiguration(BaseModel):
+    class TaskConfiguration(AbstractTaskConfiguration):
         name: str
-        utc_difference: int
         migration_task_type: str
         open_requests_file: FileDefinition
         starting_row: Optional[int] = 1
@@ -42,7 +40,6 @@ class RequestsMigrator(MigrationTaskBase):
     ):
         csv.register_dialect("tsv", delimiter="\t")
         self.migration_report = MigrationReport()
-        self.start_datetime = datetime.now(timezone.utc)
         self.valid_legacy_requests = []
         super().__init__(library_config, task_configuration)
         self.circulation_helper = CirculationHelper(
@@ -50,6 +47,19 @@ class RequestsMigrator(MigrationTaskBase):
             "",
             self.migration_report,
         )
+        try:
+            logging.info("Attempting to retrieve tenant timezone configuration...")
+            my_path = (
+                "/configurations/entries?query=(module==ORG%20and%20configName==localeSettings)"
+            )
+            self.tenant_timezone_str = json.loads(
+                self.folio_client.folio_get_single_object(my_path)["configs"][0]["value"]
+            )["timezone"]
+            logging.info("Tenant timezone is: %s", self.tenant_timezone_str)
+        except Exception:
+            logging.info('Tenant locale settings not available. Using "UTC".')
+            self.tenant_timezone_str = "UTC"
+        self.tenant_timezone = ZoneInfo(self.tenant_timezone_str)
         with open(
             self.folder_structure.legacy_records_folder
             / task_configuration.open_requests_file.file_name,
@@ -140,10 +150,7 @@ class RequestsMigrator(MigrationTaskBase):
                 res, legacy_request = self.prepare_legacy_request(legacy_request)
                 if res:
                     if self.circulation_helper.create_request(
-                        self.folio_client,
-                        legacy_request,
-                        self.migration_report,
-                        self.library_configuration.folio_release,
+                        self.folio_client, legacy_request, self.migration_report
                     ):
                         self.migration_report.add_general_statistics(
                             "Successfully migrated requests"
@@ -165,13 +172,17 @@ class RequestsMigrator(MigrationTaskBase):
                 sys.exit(1)
             if num_requests % 10 == 0:
                 logging.info(f"{timings(self.t0, t0_migration, num_requests)} {num_requests}")
+        logging.info(f"{timings(self.t0, t0_migration, num_requests)} {num_requests}")
 
     def wrap_up(self):
+        self.extradata_writer.flush()
         self.write_failed_request_to_file()
 
         with open(self.folder_structure.migration_reports_file, "w+") as report_file:
-            report_file.write("# Requests migration results   \n")
-            self.migration_report.write_migration_report(report_file, self.start_datetime)
+            self.migration_report.write_migration_report(
+                "Requests migration report", report_file, self.start_datetime
+            )
+        self.clean_out_empty_logs()
 
     def write_failed_request_to_file(self):
         csv_columns = [
@@ -239,7 +250,7 @@ class RequestsMigrator(MigrationTaskBase):
             try:
                 legacy_request = LegacyRequest(
                     legacy_request_dict,
-                    self.task_configuration.utc_difference,
+                    self.tenant_timezone,
                     legacy_reques_count,
                 )
                 if any(legacy_request.errors):
