@@ -9,8 +9,21 @@ class ReportSection(Enum):
 
 
 class Blurbs:
+    AuthoritySources = ("Authorization sources and related information", "")
+    FieldMappingDetails = ("Mapping details", "")
+    CatalogingAgency = ("Cataloging sources", "")
+    Trivia = ("Trivia", "")
     LeaderManipulation = ("Leader manipulation", "")
     TermsMapping = ("Terms Mapping", "Reference data mapping for Terms.")
+    CategoriesMapping = (
+        "Organization Contact Information Categories Mapping",
+        "Reference data mapping for Categories assigned to Organization contacts, addresses, emails, and phones numbers.",
+    )
+    OrganizationTypeMapping = (
+        "Organization Type Mapping",
+        "Reference data mapping for Organization types.",
+    )
+
     DateTimeConversions = (
         "DateTime conversions",
         "Some date and date time strings are converted to UTC DateTime objects and then printed accoding to ISO standard.",
@@ -44,7 +57,11 @@ class Blurbs:
     )
     RecordStatus = (
         "Record status (leader pos 5)",
-        "Library action: **Consider fixing d-values before migration**<br/>An overview of the Record statuses (Leader position 5) present in your source data.    Pay attention to the number of occurrences of the value 'd'. These d's are expressing that they are deleted, and the records might not work as expected in FOLIO. Consider marking them as suppressed in your current system and export them as a separate batch in order to have them suppressed in FOLIO. Allowed values according to the MARC standard are a,c,d,n,p",
+        "Library action: **All values that are not a, c, d, n or p will be set to c. If this is not what you want, you need to correct these values in your system. **<br/>An overview of the Record statuses (Leader position 5) present in your source data.    Pay attention to the number of occurrences of the value 'd'. These d's are expressing that they are deleted, and the records might not work as expected in FOLIO. Consider marking them as suppressed in your current system and export them as a separate batch in order to have them suppressed in FOLIO. Allowed values according to the MARC standard are a,c,d,n,p",
+    )
+    Suppression = (
+        "Suppression",
+        "What records got assigned what suppression setting in the records.",
     )
     ValueErrors = (
         "Records failed to migrate due to Value errors found in Transformation",
@@ -143,7 +160,7 @@ class Blurbs:
     Section3 = ("__Section 3: items", "The entries below seem to be related to items")
     Exceptions = ("Exceptions", "")
     HridHandling = (
-        "HRID Handling",
+        "HRID and 001/035 handling",
         "There are two ways of handling HRIDs. The default behaviour is to take the current 001 and move that to a new 035. This will also emerge as an Identifier on the Inventory Instances. The 001 and Instance HRID will be generated from the HRID settings in FOLIO. The second option is to maintain the 001s in the records, and also add this as the Instance HRID",
     )
     PrecedingSuccedingTitles = (
@@ -176,7 +193,7 @@ class Blurbs:
     )
     GeneralStatistics = (
         "General statistics",
-        "A list of general counterts to outline the transformation as a whole.",
+        "A list of general counters to outline the transformation as a whole.",
     )
     MappedPublisherRoleFromIndicator2 = (
         "Mapped publisher role from Indicator2",
